# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class BackendServerApiProductModelsCatalogClientModelsTableMetadata(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'catalog_name': 'str',
        'schema_name': 'str',
        'table_type': 'str',
        'data_source_format': 'str',
        'description': 'str',
        'owner': 'str',
        'created_at': 'int',
        'updated_at': 'int',
        'full_name': 'str',
        'columns': 'list[ColumnInfo]',
        'storage_location': 'str',
        'properties': 'object',
        'version': 'str',
        'row_count': 'int',
        'size_bytes': 'int'
    }

    attribute_map = {
        'name': 'name',
        'catalog_name': 'catalog_name',
        'schema_name': 'schema_name',
        'table_type': 'table_type',
        'data_source_format': 'data_source_format',
        'description': 'description',
        'owner': 'owner',
        'created_at': 'created_at',
        'updated_at': 'updated_at',
        'full_name': 'full_name',
        'columns': 'columns',
        'storage_location': 'storage_location',
        'properties': 'properties',
        'version': 'version',
        'row_count': 'row_count',
        'size_bytes': 'size_bytes'
    }

    def __init__(self, name=None, catalog_name=None, schema_name=None, table_type=None, data_source_format=None, description=None, owner=None, created_at=0, updated_at=0, full_name=None, columns=None, storage_location=None, properties=None, version=None, row_count=None, size_bytes=None, local_vars_configuration=None):  # noqa: E501
        """BackendServerApiProductModelsCatalogClientModelsTableMetadata - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._catalog_name = None
        self._schema_name = None
        self._table_type = None
        self._data_source_format = None
        self._description = None
        self._owner = None
        self._created_at = None
        self._updated_at = None
        self._full_name = None
        self._columns = None
        self._storage_location = None
        self._properties = None
        self._version = None
        self._row_count = None
        self._size_bytes = None
        self.discriminator = None

        self.name = name
        self.catalog_name = catalog_name
        self.schema_name = schema_name
        self.table_type = table_type
        if data_source_format is not None:
            self.data_source_format = data_source_format
        if description is not None:
            self.description = description
        if owner is not None:
            self.owner = owner
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if full_name is not None:
            self.full_name = full_name
        if columns is not None:
            self.columns = columns
        if storage_location is not None:
            self.storage_location = storage_location
        if properties is not None:
            self.properties = properties
        if version is not None:
            self.version = version
        if row_count is not None:
            self.row_count = row_count
        if size_bytes is not None:
            self.size_bytes = size_bytes

    @property
    def name(self):
        """Gets the name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Name of the table  # noqa: E501

        :return: The name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Name of the table  # noqa: E501

        :param name: The name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def catalog_name(self):
        """Gets the catalog_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Name of the parent catalog  # noqa: E501

        :return: The catalog_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._catalog_name

    @catalog_name.setter
    def catalog_name(self, catalog_name):
        """Sets the catalog_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Name of the parent catalog  # noqa: E501

        :param catalog_name: The catalog_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and catalog_name is None:  # noqa: E501
            raise ValueError("Invalid value for `catalog_name`, must not be `None`")  # noqa: E501

        self._catalog_name = catalog_name

    @property
    def schema_name(self):
        """Gets the schema_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Name of the parent schema  # noqa: E501

        :return: The schema_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._schema_name

    @schema_name.setter
    def schema_name(self, schema_name):
        """Sets the schema_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Name of the parent schema  # noqa: E501

        :param schema_name: The schema_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and schema_name is None:  # noqa: E501
            raise ValueError("Invalid value for `schema_name`, must not be `None`")  # noqa: E501

        self._schema_name = schema_name

    @property
    def table_type(self):
        """Gets the table_type of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Type of table (e.g., 'MANAGED', 'EXTERNAL', 'VIEW')  # noqa: E501

        :return: The table_type of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._table_type

    @table_type.setter
    def table_type(self, table_type):
        """Sets the table_type of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Type of table (e.g., 'MANAGED', 'EXTERNAL', 'VIEW')  # noqa: E501

        :param table_type: The table_type of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and table_type is None:  # noqa: E501
            raise ValueError("Invalid value for `table_type`, must not be `None`")  # noqa: E501

        self._table_type = table_type

    @property
    def data_source_format(self):
        """Gets the data_source_format of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Data format of the table (e.g., 'DELTA', 'PARQUET')  # noqa: E501

        :return: The data_source_format of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._data_source_format

    @data_source_format.setter
    def data_source_format(self, data_source_format):
        """Sets the data_source_format of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Data format of the table (e.g., 'DELTA', 'PARQUET')  # noqa: E501

        :param data_source_format: The data_source_format of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._data_source_format = data_source_format

    @property
    def description(self):
        """Gets the description of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Description or comment for the table  # noqa: E501

        :return: The description of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Description or comment for the table  # noqa: E501

        :param description: The description of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def owner(self):
        """Gets the owner of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Owner of the table  # noqa: E501

        :return: The owner of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._owner

    @owner.setter
    def owner(self, owner):
        """Sets the owner of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Owner of the table  # noqa: E501

        :param owner: The owner of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._owner = owner

    @property
    def created_at(self):
        """Gets the created_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Creation timestamp in milliseconds since epoch  # noqa: E501

        :return: The created_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: int
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Creation timestamp in milliseconds since epoch  # noqa: E501

        :param created_at: The created_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: int
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Last update timestamp in milliseconds since epoch  # noqa: E501

        :return: The updated_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: int
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Last update timestamp in milliseconds since epoch  # noqa: E501

        :param updated_at: The updated_at of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: int
        """

        self._updated_at = updated_at

    @property
    def full_name(self):
        """Gets the full_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Fully qualified name of the table  # noqa: E501

        :return: The full_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._full_name

    @full_name.setter
    def full_name(self, full_name):
        """Sets the full_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Fully qualified name of the table  # noqa: E501

        :param full_name: The full_name of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._full_name = full_name

    @property
    def columns(self):
        """Gets the columns of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        List of columns in the table  # noqa: E501

        :return: The columns of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: list[ColumnInfo]
        """
        return self._columns

    @columns.setter
    def columns(self, columns):
        """Sets the columns of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        List of columns in the table  # noqa: E501

        :param columns: The columns of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: list[ColumnInfo]
        """

        self._columns = columns

    @property
    def storage_location(self):
        """Gets the storage_location of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Physical storage location of the table data  # noqa: E501

        :return: The storage_location of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._storage_location

    @storage_location.setter
    def storage_location(self, storage_location):
        """Sets the storage_location of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Physical storage location of the table data  # noqa: E501

        :param storage_location: The storage_location of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._storage_location = storage_location

    @property
    def properties(self):
        """Gets the properties of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Additional table properties  # noqa: E501

        :return: The properties of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: object
        """
        return self._properties

    @properties.setter
    def properties(self, properties):
        """Sets the properties of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Additional table properties  # noqa: E501

        :param properties: The properties of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: object
        """

        self._properties = properties

    @property
    def version(self):
        """Gets the version of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Version identifier for versioned tables (e.g., Delta Lake)  # noqa: E501

        :return: The version of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Version identifier for versioned tables (e.g., Delta Lake)  # noqa: E501

        :param version: The version of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: str
        """

        self._version = version

    @property
    def row_count(self):
        """Gets the row_count of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Number of rows in the table  # noqa: E501

        :return: The row_count of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: int
        """
        return self._row_count

    @row_count.setter
    def row_count(self, row_count):
        """Sets the row_count of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Number of rows in the table  # noqa: E501

        :param row_count: The row_count of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: int
        """

        self._row_count = row_count

    @property
    def size_bytes(self):
        """Gets the size_bytes of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501

        Size of the table data in bytes  # noqa: E501

        :return: The size_bytes of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :rtype: int
        """
        return self._size_bytes

    @size_bytes.setter
    def size_bytes(self, size_bytes):
        """Sets the size_bytes of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.

        Size of the table data in bytes  # noqa: E501

        :param size_bytes: The size_bytes of this BackendServerApiProductModelsCatalogClientModelsTableMetadata.  # noqa: E501
        :type: int
        """

        self._size_bytes = size_bytes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BackendServerApiProductModelsCatalogClientModelsTableMetadata):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, BackendServerApiProductModelsCatalogClientModelsTableMetadata):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ClusterOperation(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'completed': 'bool',
        'progress': 'OperationProgress',
        'result': 'OperationResult',
        'cluster_id': 'str',
        'cluster_operation_type': 'ClusterOperationType'
    }

    attribute_map = {
        'id': 'id',
        'completed': 'completed',
        'progress': 'progress',
        'result': 'result',
        'cluster_id': 'cluster_id',
        'cluster_operation_type': 'cluster_operation_type'
    }

    def __init__(self, id=None, completed=None, progress=None, result=None, cluster_id=None, cluster_operation_type=None, local_vars_configuration=None):  # noqa: E501
        """ClusterOperation - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._completed = None
        self._progress = None
        self._result = None
        self._cluster_id = None
        self._cluster_operation_type = None
        self.discriminator = None

        self.id = id
        self.completed = completed
        if progress is not None:
            self.progress = progress
        if result is not None:
            self.result = result
        self.cluster_id = cluster_id
        self.cluster_operation_type = cluster_operation_type

    @property
    def id(self):
        """Gets the id of this ClusterOperation.  # noqa: E501

        ID of this operation.  # noqa: E501

        :return: The id of this ClusterOperation.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ClusterOperation.

        ID of this operation.  # noqa: E501

        :param id: The id of this ClusterOperation.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def completed(self):
        """Gets the completed of this ClusterOperation.  # noqa: E501

        Boolean indicating if this operation is completed.  # noqa: E501

        :return: The completed of this ClusterOperation.  # noqa: E501
        :rtype: bool
        """
        return self._completed

    @completed.setter
    def completed(self, completed):
        """Sets the completed of this ClusterOperation.

        Boolean indicating if this operation is completed.  # noqa: E501

        :param completed: The completed of this ClusterOperation.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and completed is None:  # noqa: E501
            raise ValueError("Invalid value for `completed`, must not be `None`")  # noqa: E501

        self._completed = completed

    @property
    def progress(self):
        """Gets the progress of this ClusterOperation.  # noqa: E501

        Details about the progress of this operation at the time of the request.             This will be absent for completed operations.  # noqa: E501

        :return: The progress of this ClusterOperation.  # noqa: E501
        :rtype: OperationProgress
        """
        return self._progress

    @progress.setter
    def progress(self, progress):
        """Sets the progress of this ClusterOperation.

        Details about the progress of this operation at the time of the request.             This will be absent for completed operations.  # noqa: E501

        :param progress: The progress of this ClusterOperation.  # noqa: E501
        :type: OperationProgress
        """

        self._progress = progress

    @property
    def result(self):
        """Gets the result of this ClusterOperation.  # noqa: E501

        The result of this operation after it has completed.             This is always provided when the operation is complete.  # noqa: E501

        :return: The result of this ClusterOperation.  # noqa: E501
        :rtype: OperationResult
        """
        return self._result

    @result.setter
    def result(self, result):
        """Sets the result of this ClusterOperation.

        The result of this operation after it has completed.             This is always provided when the operation is complete.  # noqa: E501

        :param result: The result of this ClusterOperation.  # noqa: E501
        :type: OperationResult
        """

        self._result = result

    @property
    def cluster_id(self):
        """Gets the cluster_id of this ClusterOperation.  # noqa: E501

        ID of the Cluster that is being updated.  # noqa: E501

        :return: The cluster_id of this ClusterOperation.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this ClusterOperation.

        ID of the Cluster that is being updated.  # noqa: E501

        :param cluster_id: The cluster_id of this ClusterOperation.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cluster_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cluster_id`, must not be `None`")  # noqa: E501

        self._cluster_id = cluster_id

    @property
    def cluster_operation_type(self):
        """Gets the cluster_operation_type of this ClusterOperation.  # noqa: E501

        The variety of operation being performed:             start sets the Cluster's goal state to Running,             terminate sets the Cluster's goal state to Terminated  # noqa: E501

        :return: The cluster_operation_type of this ClusterOperation.  # noqa: E501
        :rtype: ClusterOperationType
        """
        return self._cluster_operation_type

    @cluster_operation_type.setter
    def cluster_operation_type(self, cluster_operation_type):
        """Sets the cluster_operation_type of this ClusterOperation.

        The variety of operation being performed:             start sets the Cluster's goal state to Running,             terminate sets the Cluster's goal state to Terminated  # noqa: E501

        :param cluster_operation_type: The cluster_operation_type of this ClusterOperation.  # noqa: E501
        :type: ClusterOperationType
        """
        if self.local_vars_configuration.client_side_validation and cluster_operation_type is None:  # noqa: E501
            raise ValueError("Invalid value for `cluster_operation_type`, must not be `None`")  # noqa: E501

        self._cluster_operation_type = cluster_operation_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ClusterOperation):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ClusterOperation):
            return True

        return self.to_dict() != other.to_dict()



from copy import deepcopy
from datetime import date, datetime
import sys
from typing import Dict, Iterable, List, Optional, Tuple, TypeVar

import click
import colorama
from rich.table import Table

from anyscale._private.workload import WorkloadConfig
from anyscale.cli_logger import BlockLogger


logger = BlockLogger()


class AnyscaleCommand(click.Command):
    """
    AnyscaleCommand is a subclass of click.Command that allows for the addition of
    CLI examples.

    :param __doc_cli_example__: A string that represents CLI examples.
    """

    def __init__(self, *args, **kwargs):
        self.__doc_cli_example__ = kwargs.pop("example", None)
        self.__is_alpha__ = kwargs.pop("is_alpha", False)
        self.__is_beta__ = kwargs.pop("is_beta", False)
        super().__init__(*args, **kwargs)

    @property
    def is_alpha(self) -> bool:
        return self.__is_alpha__

    @property
    def is_beta(self) -> bool:
        return self.__is_beta__


class LegacyAnyscaleCommand(click.Command):
    """
    LegacyAnyscaleCommand is a subclass of click.Command that allows for the addition of
    upgrade information.

    :param __doc_new_cli__: The CLI command that the legacy command should be upgraded to.
    :param __doc_new_prefix__ : The prefix of the new CLI command.
    """

    def __init__(self, *args, **kwargs):
        self.__doc_new_cli__ = kwargs.pop("new_cli", None)
        self.__doc_new_prefix__ = kwargs.pop("new_prefix", None)
        self.__doc_legacy_prefix__ = kwargs.pop("legacy_prefix", None)
        self.__is_limited_support__ = kwargs.pop("is_limited_support", False)
        super().__init__(*args, **kwargs)

    def get_new_cli(self) -> AnyscaleCommand:
        return self.__doc_new_cli__

    def get_new_prefix(self) -> str:
        return self.__doc_new_prefix__

    def get_legacy_prefix(self) -> str:
        return self.__doc_legacy_prefix__

    def is_limited_support(self) -> bool:
        return self.__is_limited_support__


# Take from https://stackoverflow.com/questions/51846634/click-dynamic-defaults-for-prompts-based-on-other-options
class OptionPromptNull(click.Option):
    """
    Option class that allows default values based on previous params
    """

    _value_key = "_default_val"

    def __init__(self, *args, **kwargs):
        self.default_option = kwargs.pop("default_option", None)
        super().__init__(*args, **kwargs)

    def get_default(self, ctx, **kwargs):
        if not hasattr(self, self._value_key):
            if self.default_option is None:
                default = super().get_default(ctx, **kwargs)
            else:
                arg = ctx.params.get(self.default_option)
                default = (
                    self.type_cast_value(ctx, self.default(arg))
                    if arg is not None
                    else None
                )
            setattr(self, self._value_key, default)
        return getattr(self, self._value_key)


# Taken from https://stackoverflow.com/questions/44247099/click-command-line-interfaces-make-options-required-if-other-optional-option-is
class NotRequiredIf(click.Option):
    """
    Option class that allows an option to be not required if a separate option is available.
    """

    def __init__(self, *args, **kwargs):
        self.not_required_if = kwargs.pop("not_required_if")
        assert self.not_required_if, "'not_required_if' parameter required"
        kwargs["help"] = (
            kwargs.get("help", "")
            + " NOTE: This argument is mutually exclusive with %s"
            % self.not_required_if
        ).strip()
        super().__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        we_are_present = self.name in opts
        other_present = self.not_required_if in opts

        if other_present:
            if we_are_present:
                raise click.UsageError(
                    "Illegal usage: `%s` is mutually exclusive with `%s`"
                    % (self.name, self.not_required_if)
                )
            else:
                self.prompt = None
                self.default = None

        return super().handle_parse_result(ctx, opts, args)


def convert_kv_strings_to_dict(strings: Tuple[str]) -> Dict[str, str]:
    """Convert args/env_vars of the form "key=val" into a dictionary of {key: val}.

    NOTE(edoakes): this mimics the functionality of the `serve run` CLI and should be
    kept in sync with it.
    """
    ret_dict = {}
    for s in strings:
        split = s.split("=", maxsplit=1)
        if len(split) != 2 or len(split[1]) == 0:
            raise click.ClickException(
                f"Invalid key-value string '{s}'. Must be of the form 'key=value'."
            )

        ret_dict[split[0]] = split[1]

    return ret_dict


T = TypeVar("T", bound=WorkloadConfig)


def override_env_vars(config: T, overrides: Dict[str, str]) -> T:
    """Returns a new copy of the WorkloadConfig with env vars overridden.

    This is a per-key override, so keys already in the config that are not specified in
    overrides will not be updated.
    """
    if not overrides:
        return config

    final_env_vars = deepcopy(config.env_vars) if config.env_vars else {}
    final_env_vars.update(overrides)
    return config.options(env_vars=final_env_vars)


def parse_repeatable_tags_to_dict(strings: Iterable[str]) -> Dict[str, List[str]]:
    """Parse repeatable --tag args into dict[key] -> list[values].

    Accepts both "key:value" and "key=value". Ignores malformed entries.
    Values for the same key are ORed; different keys are ANDed by the backend.
    """
    result: Dict[str, List[str]] = {}
    for raw in strings or []:
        if ":" in raw:
            key, value = raw.split(":", 1)
        elif "=" in raw:
            key, value = raw.split("=", 1)
        else:
            continue
        key = key.strip()
        value = value.strip()
        if not key or not value:
            continue
        result.setdefault(key, []).append(value)
    return result


def normalize_tags_to_api_list(strings: Iterable[str]) -> List[str]:
    """Normalize repeatable --tag args into API wire format list[str] "key:value".

    Accepts both "key:value" and "key=value". Ignores malformed entries.
    """
    flattened: List[str] = []
    for raw in strings or []:
        if ":" in raw:
            key, value = raw.split(":", 1)
        elif "=" in raw:
            key, value = raw.split("=", 1)
        else:
            continue
        key = key.strip()
        value = value.strip()
        if not key or not value:
            continue
        flattened.append(f"{key}:{value}")
    return flattened


def flatten_tag_dict_to_api_list(
    tags: Optional[Dict[str, List[str]]]
) -> Optional[List[str]]:
    """Flatten dict[key] -> list[values] into list[str] "key:value" for API.

    Returns None if input is None or empty after normalization.
    """
    if not tags:
        return None
    out: List[str] = []
    for key, values in tags.items():
        if not key:
            continue
        for value in values or []:
            if value:
                out.append(f"{key}:{value}")
    return out if out else None


def parse_tags_kv_to_str_map(pairs: Iterable[str]) -> Dict[str, str]:
    """Parse repeatable key=value (or key:value) into a simple {key: value} map.

    Last occurrence wins for duplicate keys. Malformed entries are ignored.
    """
    result: Dict[str, str] = {}
    for raw in pairs or []:
        if ":" in raw:
            key, value = raw.split(":", 1)
        elif "=" in raw:
            key, value = raw.split("=", 1)
        else:
            continue
        key = key.strip()
        value = value.strip()
        if not key or not value:
            continue
        result[key] = value
    return result


def build_kv_table(
    pairs: Iterable[Tuple[str, str]], *, title: Optional[str] = None
) -> Table:
    """Build a Rich table for key/value pairs.

    - Sorts rows by key then value for stable output
    - Columns wrap to fit smaller terminals
    """
    table = Table(show_header=True, header_style="bold", expand=False, title=title)
    table.add_column("KEY", overflow="fold")
    table.add_column("VALUE", overflow="fold")
    sorted_pairs = sorted(
        [(str(k), str(v)) for k, v in (pairs or [])], key=lambda kv: (kv[0], kv[1])
    )
    for key, value in sorted_pairs:
        table.add_row(key, value)
    return table


class DeprecatedAnyscaleCommand(click.Command):
    """
    DeprecatedAnyscaleCommand is a subclass of click.Command that shows deprecation warnings.

    Similar to LegacyAnyscaleCommand but focuses on deprecation with dates and alternatives.
    """

    def __init__(self, *args, **kwargs):
        self.__removal_date__ = kwargs.pop("removal_date", None)
        self.__deprecation_message__ = kwargs.pop("deprecation_message", None)
        self.__alternative__ = kwargs.pop("alternative", None)
        super().__init__(*args, **kwargs)

    def get_help(self, ctx):
        """Override get_help to show deprecation warning when help is displayed."""
        self._show_deprecation_warning()
        return super().get_help(ctx)

    def invoke(self, ctx):
        """Override invoke to show deprecation warning before executing."""
        self._show_deprecation_warning()
        return super().invoke(ctx)

    def _show_deprecation_warning(self):
        """Show the deprecation warning."""

        # Visual separator for attention
        print("\n" + "=" * 80, file=sys.stderr)
        print(
            f"{colorama.Style.BRIGHT}{colorama.Fore.YELLOW}⚠️  DEPRECATION WARNING ⚠️{colorama.Style.RESET_ALL}",
            file=sys.stderr,
        )
        print("=" * 80, file=sys.stderr)

        # Build deprecation message
        base_msg = (
            self.__deprecation_message__
            if self.__deprecation_message__
            else f"Command '{self.name}' is deprecated"
        )

        # Removal date information with grammar-aware connector
        date_msg = None
        if self.__removal_date__:
            date_str = self._format_removal_date(self.__removal_date__)
            if date_str:
                ends_with_punct = base_msg.strip().endswith((".", "!", "?"))
                if ends_with_punct:
                    date_msg = f"It will be removed on {date_str}"
                else:
                    date_msg = f"and will be removed on {date_str}"

        # Alternative suggestion
        alternative_msg = None
        if self.__alternative__:
            alternative_msg = f"\n\n➡️  {colorama.Style.BRIGHT}Please {self.__alternative__}{colorama.Style.RESET_ALL}"

        main_line_parts = [part for part in (base_msg, date_msg) if part]
        deprecation_msg = " ".join(main_line_parts)
        if alternative_msg:
            deprecation_msg += alternative_msg

        # Logger warning but also print directly for visibility
        print(
            f"\n{colorama.Fore.YELLOW}{deprecation_msg}{colorama.Style.RESET_ALL}",
            file=sys.stderr,
        )
        print("=" * 80 + "\n", file=sys.stderr)

    def _format_removal_date(self, removal_date) -> Optional[str]:
        """Format the removal date for display."""
        try:
            if isinstance(removal_date, str):
                parsed_date = datetime.strptime(removal_date, "%Y-%m-%d").date()
            elif isinstance(removal_date, datetime):
                parsed_date = removal_date.date()
            elif isinstance(removal_date, date):
                parsed_date = removal_date
            else:
                return None

            return parsed_date.strftime("%Y-%m-%d")
        except (ValueError, AttributeError):
            return str(removal_date)



from dataclasses import dataclass, field
from datetime import datetime
from typing import ClassVar, Dict, Optional, Union

from anyscale._private.models.model_base import ModelBase, ModelEnum


class ImageBuildStatus(ModelEnum):
    """Status of an image build operation."""

    IN_PROGRESS = "IN_PROGRESS"
    SUCCEEDED = "SUCCEEDED"
    FAILED = "FAILED"
    UNKNOWN = "UNKNOWN"

    def __str__(self):
        return self.name

    __docstrings__: ClassVar[Dict[str, str]] = {
        IN_PROGRESS: "The image build is in progress.",
        SUCCEEDED: "The image build succeeded.",
        FAILED: "The image build failed.",
        UNKNOWN: "The CLI/SDK received an unexpected state from the API server. In most cases, this means you need to update the CLI.",
    }


@dataclass(frozen=True)
class ImageBuild(ModelBase):
    __doc_py_example__ = """\
import anyscale
from anyscale.image.models import ImageBuild

first_image: ImageBuild = next(anyscale.image.list(max_items=1), None)
"""

    id: str = field(metadata={"docstring": "Unique identifier of the image."},)

    def _validate_id(self, id: str):  # noqa: A002, A003
        if id is None or not isinstance(id, str):
            raise ValueError("The image id must be a string.")

    name: str = field(
        metadata={
            "docstring": "Human-readable name assigned to the image (cluster environment)."
        },
    )

    def _validate_name(self, name: str):
        if name is None or not isinstance(name, str):
            raise ValueError("The image name must be a string.")

    project_id: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "Project identifier that owns this image, if available."
        },
    )

    def _validate_project_id(self, project_id: Optional[str]):
        if project_id is not None and not isinstance(project_id, str):
            raise ValueError("The project id must be a string if provided.")

    creator_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "Identifier of the user who created the image."},
    )

    def _validate_creator_id(self, creator_id: Optional[str]):
        if creator_id is not None and not isinstance(creator_id, str):
            raise ValueError("The creator id must be a string if provided.")

    creator_email: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "Email address of the user who created the image, if available."
        },
    )

    def _validate_creator_email(self, creator_email: Optional[str]):
        if creator_email is not None and not isinstance(creator_email, str):
            raise ValueError("The creator email must be a string if provided.")

    is_anonymous: bool = field(
        default=False,
        metadata={
            "docstring": "Whether the image was created as an anonymous (workspace-scoped) resource."
        },
    )

    def _validate_is_anonymous(self, is_anonymous: bool):
        if not isinstance(is_anonymous, bool):
            raise ValueError("The is_anonymous flag must be a boolean.")

    created_at: Optional[datetime] = field(
        default=None, metadata={"docstring": "Timestamp when the image was created."},
    )

    def _validate_created_at(self, created_at: Optional[datetime]):
        if created_at is not None and not isinstance(created_at, datetime):
            raise ValueError("The created_at field must be a datetime if provided.")

    last_modified_at: Optional[datetime] = field(
        default=None,
        metadata={"docstring": "Timestamp when the image was last updated."},
    )

    def _validate_last_modified_at(self, last_modified_at: Optional[datetime]):
        if last_modified_at is not None and not isinstance(last_modified_at, datetime):
            raise ValueError(
                "The last_modified_at field must be a datetime if provided."
            )

    latest_build_id: Optional[str] = field(
        default=None,
        metadata={"docstring": "Identifier of the most recent build for this image."},
    )

    def _validate_latest_build_id(self, latest_build_id: Optional[str]):
        if latest_build_id is not None and not isinstance(latest_build_id, str):
            raise ValueError("The latest build id must be a string if provided.")

    latest_build_revision: Optional[int] = field(
        default=None,
        metadata={
            "docstring": "Revision number of the most recent image build (treated as the latest version)."
        },
    )

    def _validate_latest_build_revision(self, latest_build_revision: Optional[int]):
        if latest_build_revision is not None and not isinstance(
            latest_build_revision, int
        ):
            raise ValueError(
                "The latest build revision must be an integer if provided."
            )

    latest_build_status: Optional[Union[str, ImageBuildStatus]] = field(
        default=None, metadata={"docstring": "Status of the most recent image build."},
    )

    def _validate_latest_build_status(
        self, latest_build_status: Optional[Union[str, ImageBuildStatus]]
    ) -> Optional[ImageBuildStatus]:
        if latest_build_status is None:
            return None
        if isinstance(latest_build_status, ImageBuildStatus):
            return latest_build_status
        return ImageBuildStatus.validate(str(latest_build_status).upper())

    latest_image_uri: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "URI for the latest published image version, if available."
        },
    )

    def _validate_latest_image_uri(self, latest_image_uri: Optional[str]):
        if latest_image_uri is not None and not isinstance(latest_image_uri, str):
            raise ValueError("The latest image URI must be a string if provided.")



"""
Azure Resource Manager (ARM) utility functions for deploying templates.

This module provides utilities for ARM template deployment operations
including creation, waiting, and output retrieval.
"""

import time
from typing import Any, Dict, Optional, Tuple

from azure.core.exceptions import HttpResponseError, ResourceNotFoundError
from azure.identity import DefaultAzureCredential
from azure.mgmt.resource import ResourceManagementClient
from azure.mgmt.resource.resources.models import (
    Deployment,
    DeploymentMode,
    DeploymentProperties,
)
from click import ClickException

from anyscale.cli_logger import BlockLogger


class ARMTemplateUtils:
    """Utility class for Azure Resource Manager template operations."""

    def __init__(self, subscription_id: str, logger: Optional[BlockLogger] = None):
        """
        Initialize ARM template utilities for a specific subscription.

        Args:
            subscription_id: Azure subscription ID
            logger: Optional BlockLogger for output
        """
        self.log = logger or BlockLogger()
        self.subscription_id = subscription_id
        self._credential = None
        self._client = None

    def _get_credential(self) -> DefaultAzureCredential:
        """Get or create Azure credential."""
        if self._credential is None:
            self._credential = DefaultAzureCredential()
        return self._credential

    def _get_client(self) -> ResourceManagementClient:
        """Get or create Resource Management client."""
        if self._client is None:
            credential = self._get_credential()
            self._client = ResourceManagementClient(credential, self.subscription_id)
        return self._client

    def deploy_template(
        self,
        deployment_name: str,
        resource_group: str,
        template: Dict[str, Any],
        parameters: Dict[str, Any],
        timeout_seconds: int = 600,
        mode: DeploymentMode = DeploymentMode.INCREMENTAL,
    ) -> Tuple[Dict[str, Any], str]:
        """
        Deploy an ARM template and wait for completion.

        Args:
            deployment_name: Name of the deployment
            resource_group: Resource group name
            template: ARM template as dictionary
            parameters: Template parameters
            timeout_seconds: Timeout in seconds (default: 600)
            mode: Deployment mode (default: Incremental)

        Returns:
            Tuple containing dictionary of output key-value pairs and portal URL

        Raises:
            ClickException: If deployment fails or times out
        """
        client = self._get_client()

        # Format parameters for ARM (ARM expects {"key": {"value": "val"}})
        arm_parameters = {k: {"value": v} for k, v in parameters.items()}

        deployment_properties = DeploymentProperties(
            mode=mode, template=template, parameters=arm_parameters
        )

        self.log.info(f"Starting ARM deployment: {deployment_name}")

        try:
            # Begin deployment
            deployment_operation = client.deployments.begin_create_or_update(
                resource_group_name=resource_group,
                deployment_name=deployment_name,
                parameters=Deployment(properties=deployment_properties),
            )

            # Wait for completion
            return self._wait_for_deployment(
                client,
                resource_group,
                deployment_name,
                deployment_operation,
                timeout_seconds,
            )

        except HttpResponseError as e:
            raise ClickException(f"Failed to start ARM deployment: {e.message}")

    def _wait_for_deployment(
        self,
        client: ResourceManagementClient,
        resource_group: str,
        deployment_name: str,
        deployment_operation,
        timeout_seconds: int,
    ) -> Tuple[Dict[str, Any], str]:
        """Wait for ARM deployment to complete with progress tracking."""
        portal_url = (
            f"https://portal.azure.com/#blade/HubsExtension/DeploymentDetailsBlade/"
            f"overview/id/%2Fsubscriptions%2F{self.subscription_id}%2F"
            f"resourceGroups%2F{resource_group}%2Fproviders%2F"
            f"Microsoft.Resources%2Fdeployments%2F{deployment_name}"
        )
        self.log.info(f"Track progress at: {portal_url}")

        with self.log.spinner("Deploying ARM template..."):
            start_time = time.time()
            end_time = start_time + timeout_seconds

            while time.time() < end_time:
                try:
                    # Check if operation is done
                    if deployment_operation.done():
                        result = deployment_operation.result()

                        if result.properties.provisioning_state == "Succeeded":
                            self.log.info(
                                f"ARM deployment {deployment_name} completed successfully"
                            )
                            return self._extract_outputs(result), portal_url

                        # Deployment failed
                        error_msg = self._get_deployment_errors(
                            client, resource_group, deployment_name
                        )
                        raise ClickException(
                            f"ARM deployment failed with state: "
                            f"{result.properties.provisioning_state}. "
                            f"Errors: {error_msg}. "
                            f"Check the deployment at: {portal_url}"
                        )

                    # Still in progress, wait
                    time.sleep(5)

                except HttpResponseError as e:
                    raise ClickException(
                        f"Error checking deployment status: {e.message}"
                    )

            # Timeout
            raise ClickException(
                f"ARM deployment {deployment_name} timed out after "
                f"{timeout_seconds} seconds. Check Azure Portal: {portal_url}"
            )

    def _extract_outputs(self, deployment: Deployment) -> Dict[str, Any]:
        """Extract outputs from a completed deployment."""
        if not deployment.properties or not deployment.properties.outputs:
            return {}

        # ARM outputs have format: {"key": {"type": "string", "value": "..."}}
        # We want to return: {"key": "value"}
        outputs = {}
        for key, output_obj in deployment.properties.outputs.items():
            if isinstance(output_obj, dict) and "value" in output_obj:
                outputs[key] = output_obj["value"]
            else:
                outputs[key] = output_obj

        return outputs

    def _get_deployment_errors(
        self,
        client: ResourceManagementClient,
        resource_group: str,
        deployment_name: str,
    ) -> str:
        """Get detailed error messages from deployment operations."""
        try:
            operations = client.deployment_operations.list(
                resource_group_name=resource_group, deployment_name=deployment_name
            )

            errors = []
            for operation in operations:
                if (
                    operation.properties
                    and operation.properties.status_message
                    and operation.properties.provisioning_state == "Failed"
                ):
                    msg = operation.properties.status_message
                    if hasattr(msg, "error"):
                        error_detail = f"{msg.error.code}: {msg.error.message}"
                    else:
                        error_detail = str(msg)

                    resource_name = "Unknown"
                    if (
                        operation.properties.target_resource
                        and operation.properties.target_resource.resource_name
                    ):
                        resource_name = (
                            operation.properties.target_resource.resource_name
                        )
                    errors.append(f"Resource: {resource_name}, Error: {error_detail}")

            return "; ".join(errors) if errors else "No detailed error information"

        except Exception as e:  # noqa: BLE001
            return f"Could not retrieve error details: {e}"

    def get_deployment_outputs(
        self, deployment_name: str, resource_group: str
    ) -> Dict[str, Any]:
        """
        Get outputs from an existing ARM deployment.

        Args:
            deployment_name: Name of the deployment
            resource_group: Resource group name

        Returns:
            Dictionary of output key-value pairs

        Raises:
            ClickException: If deployment not found
        """
        client = self._get_client()

        try:
            deployment = client.deployments.get(
                resource_group_name=resource_group, deployment_name=deployment_name
            )
            return self._extract_outputs(deployment)

        except ResourceNotFoundError:
            raise ClickException(
                f"Deployment {deployment_name} not found in resource group {resource_group}"
            )
        except HttpResponseError as e:
            raise ClickException(f"Failed to get deployment outputs: {e.message}")



from io import StringIO
import json
from typing import Dict, IO, Optional

import click
from rich.console import Console
from rich.table import Table
import yaml

import anyscale
from anyscale.authenticate import get_auth_api_client
from anyscale.commands import command_examples
from anyscale.commands.list_util import (
    display_list,
    MAX_PAGE_SIZE,
    NON_INTERACTIVE_DEFAULT_MAX_ITEMS,
    validate_page_size,
)
from anyscale.commands.util import AnyscaleCommand
from anyscale.image.models import ImageBuild
from anyscale.util import AnyscaleJSONEncoder, get_endpoint, validate_non_negative_arg


@click.group(
    "image", help="Manage images to define dependencies on Anyscale.",
)
def image_cli() -> None:
    pass


def validate_max_items(ctx, param, value):
    if value is None:
        return None
    return validate_non_negative_arg(ctx, param, value)


def _create_image_list_table(show_header: bool) -> Table:
    table = Table(show_header=show_header, expand=True)
    # Allow wrapping for all columns to prevent text cutoff
    table.add_column("NAME", no_wrap=False, overflow="fold", ratio=3, min_width=15)
    table.add_column(
        "LATEST VERSION", no_wrap=False, overflow="fold", ratio=2, min_width=14,
    )
    table.add_column(
        "LATEST URI", no_wrap=False, overflow="fold", ratio=4, min_width=20,
    )
    table.add_column(
        "CREATED BY", no_wrap=False, overflow="fold", ratio=3, min_width=24,
    )
    table.add_column(
        "CREATED AT", no_wrap=False, overflow="fold", ratio=2, min_width=20,
    )
    return table


def _create_image_list_table_verbose(show_header: bool) -> Table:
    table = Table(show_header=show_header, expand=True, box=None)
    table.add_column("NAME", overflow="fold", no_wrap=False)
    table.add_column("ID", overflow="fold", no_wrap=False)
    table.add_column("PROJECT", overflow="fold", no_wrap=False)
    table.add_column("VERSION", overflow="fold", no_wrap=False)
    table.add_column("BUILD ID", overflow="fold", no_wrap=False)
    table.add_column("STATUS", overflow="fold", no_wrap=False)
    table.add_column("ANON", overflow="fold", no_wrap=False)
    table.add_column("URI", overflow="fold", no_wrap=False)
    table.add_column("CREATED BY", overflow="fold", no_wrap=False)
    table.add_column("CREATED AT", overflow="fold", no_wrap=False)
    table.add_column("MODIFIED AT", overflow="fold", no_wrap=False)
    return table


def _format_image_output(image: ImageBuild) -> Dict[str, str]:
    created_at = image.created_at.strftime("%Y-%m-%d %H:%M") if image.created_at else ""
    latest_version = (
        "" if image.latest_build_revision is None else str(image.latest_build_revision)
    )

    return {
        "name": image.name,
        "latest_version": latest_version,
        "latest_uri": image.latest_image_uri or "",
        "created_by": image.creator_email or image.creator_id or "",
        "created_at": created_at,
    }


def _format_image_output_verbose(image: ImageBuild) -> Dict[str, str]:
    created_at = image.created_at.strftime("%Y-%m-%d %H:%M") if image.created_at else ""
    last_modified = (
        image.last_modified_at.strftime("%Y-%m-%d %H:%M")
        if image.last_modified_at
        else ""
    )
    latest_version = (
        "" if image.latest_build_revision is None else str(image.latest_build_revision)
    )
    build_status = str(image.latest_build_status) if image.latest_build_status else ""

    return {
        "name": image.name,
        "id": image.id,
        "project": image.project_id or "",
        "version": latest_version,
        "build_id": image.latest_build_id or "",
        "status": build_status,
        "anon": "Y" if image.is_anonymous else "N",
        "uri": image.latest_image_uri or "",
        "created_by": image.creator_email or image.creator_id or "",
        "created_at": created_at,
        "modified_at": last_modified,
    }


@image_cli.command(
    name="build",
    help=("Build an image from a Containerfile."),
    cls=AnyscaleCommand,
    example=command_examples.IMAGE_BUILD_EXAMPLE,
)
@click.option(
    "--containerfile",
    "-f",
    help="Path to the Containerfile.",
    type=click.File("rb"),
    required=True,
)
@click.option(
    "--name",
    "-n",
    help="Name for the image. If the image with the same name already exists, a new version will be built. Otherwise, a new image will be created.",
    required=True,
    type=str,
)
@click.option(
    "--ray-version",
    "-r",
    help="The Ray version (X.Y.Z) specified for this image specified by either an image URI or a containerfile. If you don't specify a Ray version, Anyscale defaults to the latest Ray version available at the time of the Anyscale CLI/SDK release.",
    type=str,
    default=None,
)
def build(
    containerfile: IO[bytes], name: str, ray_version: Optional[str] = None
) -> None:
    try:
        containerfile_str = containerfile.read().decode("utf-8")
        image_uri = anyscale.image.build(
            containerfile_str, name=name, ray_version=ray_version
        )
        print(f"Image built successfully with URI: {image_uri}")
    except ValueError as e:
        raise click.ClickException(str(e)) from None
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(f"Failed to build image: {e}") from None


@image_cli.command(
    name="get",
    help=("Get details of an image."),
    cls=AnyscaleCommand,
    example=command_examples.IMAGE_GET_EXAMPLE,
)
@click.option(
    "--name",
    "-n",
    help=(
        "Get the details of an image.\n\n"
        "The name can contain an optional version, e.g., 'name:version'. "
        "If no version is provided, the latest one will be used.\n\n"
    ),
    type=str,
    default=None,
    required=True,
)
@click.option(
    "-j", "--json", "json_output", is_flag=True, default=False, help="Output as JSON.",
)
@click.option(
    "--yaml", "yaml_output", is_flag=True, default=False, help="Output as YAML.",
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Show all fields including IDs and metadata.",
)
def get(name: str, json_output: bool, yaml_output: bool, verbose: bool) -> None:
    try:
        image = anyscale.image.get(name=name)

        if json_output:
            print(json.dumps(image.to_dict(), indent=2, cls=AnyscaleJSONEncoder))
        elif yaml_output:
            stream = StringIO()
            yaml.safe_dump(image.to_dict(), stream, sort_keys=False)
            print(stream.getvalue(), end="")
        else:
            console = Console()

            if verbose:
                table = _create_image_list_table_verbose(show_header=True)
                formatted = _format_image_output_verbose(image)
                table.add_row(
                    formatted["name"],
                    formatted["id"],
                    formatted["project"],
                    formatted["version"],
                    formatted["build_id"],
                    formatted["status"],
                    formatted["anon"],
                    formatted["uri"],
                    formatted["created_by"],
                    formatted["created_at"],
                    formatted["modified_at"],
                )
            else:
                table = _create_image_list_table(show_header=True)
                formatted = _format_image_output(image)
                table.add_row(
                    formatted["name"],
                    formatted["latest_version"],
                    formatted["latest_uri"],
                    formatted["created_by"],
                    formatted["created_at"],
                )

            console.print(table)

    except ValueError as e:
        raise click.ClickException(str(e)) from None
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(f"Failed to get image: {e}") from None


@image_cli.command(
    name="list",
    help="List images.",
    cls=AnyscaleCommand,
    example=command_examples.IMAGE_LIST_EXAMPLE,
)
@click.option("--image-id", "--id", help="ID of the image to display.")
@click.option("--name", "-n", help="Substring to match against the image name.")
@click.option(
    "--image-name",
    help="Substring to match against the resolved image URI (BYOD images only).",
)
@click.option("--project", help="Filter images by project name.")
@click.option(
    "--created-by-me",
    is_flag=True,
    default=False,
    help="List images created by me only.",
)
@click.option(
    "--include-archived",
    is_flag=True,
    default=False,
    help="Include archived images in the results.",
)
@click.option(
    "--include-anonymous",
    is_flag=True,
    default=False,
    help="Include anonymous (workspace-scoped) images.",
)
@click.option(
    "--max-items",
    type=int,
    callback=validate_max_items,
    help="Max total items (only with --no-interactive).",
)
@click.option(
    "--page-size",
    type=int,
    default=10,
    show_default=True,
    callback=validate_page_size,
    help=f"Items per page (max {MAX_PAGE_SIZE}).",
)
@click.option(
    "--interactive/--no-interactive",
    default=True,
    show_default=True,
    help="Use interactive paging.",
)
@click.option(
    "-j",
    "--json",
    "json_output",
    is_flag=True,
    default=False,
    help="Emit structured JSON to stdout.",
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Show all fields including IDs and metadata.",
)
def list(  # noqa: A001, PLR0913
    image_id: Optional[str],
    name: Optional[str],
    image_name: Optional[str],
    project: Optional[str],
    created_by_me: bool,
    include_archived: bool,
    include_anonymous: bool,
    max_items: Optional[int],
    page_size: int,
    interactive: bool,
    json_output: bool,
    verbose: bool,
) -> None:
    if max_items is not None and interactive:
        raise click.UsageError("--max-items only allowed with --no-interactive")

    stderr = Console(stderr=True)
    effective_max = max_items
    if not interactive and effective_max is None:
        stderr.print(
            f"Defaulting to {NON_INTERACTIVE_DEFAULT_MAX_ITEMS} items in batch mode; "
            "use --max-items to override."
        )
        effective_max = NON_INTERACTIVE_DEFAULT_MAX_ITEMS

    console = Console()
    creator_id = None
    if created_by_me:
        auth_block = get_auth_api_client()
        creator_id = auth_block.api_client.get_user_info_api_v2_userinfo_get().result.id

    stderr.print("[bold]Listing images with:[/]")
    stderr.print(f"• name             = {name or '<any>'}")
    stderr.print(f"• image_uri        = {image_name or '<any>'}")
    stderr.print(f"• project          = {project or '<any>'}")
    stderr.print(f"• created_by_me    = {created_by_me}")
    stderr.print(f"• include_archived = {include_archived}")
    stderr.print(f"• include_anonymous= {include_anonymous}")
    stderr.print(f"• mode             = {'interactive' if interactive else 'batch'}")
    stderr.print(f"• per-page limit   = {page_size}")
    stderr.print(f"• max-items total  = {effective_max or 'all'}")
    stderr.print(
        f"\nView your Images in the UI at {get_endpoint('/configurations/app-config-versions')}\n"
    )

    if json_output:

        def formatter(image):
            return image.to_dict()

        table_creator = _create_image_list_table
    elif verbose:
        formatter = _format_image_output_verbose
        table_creator = _create_image_list_table_verbose
    else:
        formatter = _format_image_output
        table_creator = _create_image_list_table

    try:
        iterator = anyscale.image.list(
            image_id=image_id,
            name=name,
            image_name=image_name,
            project=project,
            creator_id=creator_id,
            include_archived=include_archived,
            include_anonymous=include_anonymous,
            max_items=None if interactive else effective_max,
            page_size=page_size,
        )
        total = display_list(
            iterator=iter(iterator),
            item_formatter=formatter,
            table_creator=table_creator,
            json_output=json_output,
            page_size=page_size,
            interactive=interactive,
            max_items=effective_max,
            console=console,
        )

        if not json_output:
            if total > 0:
                stderr.print(f"\nFetched {total} images.")
            else:
                stderr.print("\nNo images found.")
    except ValueError as e:
        raise click.ClickException(str(e)) from None
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(f"Failed to list images: {e}") from None


@image_cli.command(
    name="register",
    help=("Register a custom container image with a container image name."),
    cls=AnyscaleCommand,
    example=command_examples.IMAGE_REGISTER_EXAMPLE,
)
@click.option(
    "--image-uri",
    help="The URI of the custom container image to register.",
    type=str,
    required=True,
)
@click.option(
    "--name",
    "-n",
    help="Name for the container image. If the name already exists, a new version will be built. Otherwise, a new container image will be created.",
    required=True,
    type=str,
)
@click.option(
    "--ray-version",
    "-r",
    help="The Ray version (X.Y.Z) specified for this image specified by either an image URI or a containerfile. If you don't specify a Ray version, Anyscale defaults to the latest Ray version available at the time of the Anyscale CLI/SDK release.",
    type=str,
    default=None,
)
@click.option(
    "--registry-login-secret",
    help="Name or identifier of the secret containing credentials to authenticate to the docker registry hosting the image.",
    type=str,
    default=None,
)
def register(
    image_uri: str,
    name: str,
    ray_version: Optional[str] = None,
    registry_login_secret: Optional[str] = None,
) -> None:
    try:
        built_image_uri = anyscale.image.register(
            image_uri=image_uri,
            registry_login_secret=registry_login_secret,
            ray_version=ray_version,
            name=name,
        )
        print(f"Image registered successfully with URI: {built_image_uri}")
    except ValueError as e:
        raise click.ClickException(str(e)) from None
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(f"Failed to register image: {e}") from None


@image_cli.command(
    name="archive",
    help="Archive an image and all of its versions.",
    cls=AnyscaleCommand,
    example=command_examples.IMAGE_ARCHIVE_EXAMPLE,
)
@click.option(
    "--name",
    "-n",
    help="Name of the image to archive. Can include an optional version tag (e.g., 'name:version').",
    type=str,
    required=True,
)
def archive(name: str) -> None:
    """Archive an image.

    Once archived, the image name will no longer be usable in the organization.
    Archived images can still be viewed using --include-archived in list.
    """
    try:
        anyscale.image.archive(name=name)
        print(f"Image '{name}' archived successfully.")
    except ValueError as e:
        raise click.ClickException(str(e)) from None
    except Exception as e:  # noqa: BLE001
        raise click.ClickException(f"Failed to archive image: {e}") from None



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class Metric(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'current_value': 'float',
        'max_over_time': 'float',
        'min_over_time': 'float',
        'median_over_time': 'float',
        'rate': 'float',
        'max_rate': 'float',
        'min_rate': 'float',
        'median_rate': 'float'
    }

    attribute_map = {
        'name': 'name',
        'current_value': 'current_value',
        'max_over_time': 'max_over_time',
        'min_over_time': 'min_over_time',
        'median_over_time': 'median_over_time',
        'rate': 'rate',
        'max_rate': 'max_rate',
        'min_rate': 'min_rate',
        'median_rate': 'median_rate'
    }

    def __init__(self, name=None, current_value=None, max_over_time=None, min_over_time=None, median_over_time=None, rate=None, max_rate=None, min_rate=None, median_rate=None, local_vars_configuration=None):  # noqa: E501
        """Metric - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._current_value = None
        self._max_over_time = None
        self._min_over_time = None
        self._median_over_time = None
        self._rate = None
        self._max_rate = None
        self._min_rate = None
        self._median_rate = None
        self.discriminator = None

        self.name = name
        if current_value is not None:
            self.current_value = current_value
        if max_over_time is not None:
            self.max_over_time = max_over_time
        if min_over_time is not None:
            self.min_over_time = min_over_time
        if median_over_time is not None:
            self.median_over_time = median_over_time
        if rate is not None:
            self.rate = rate
        if max_rate is not None:
            self.max_rate = max_rate
        if min_rate is not None:
            self.min_rate = min_rate
        if median_rate is not None:
            self.median_rate = median_rate

    @property
    def name(self):
        """Gets the name of this Metric.  # noqa: E501


        :return: The name of this Metric.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Metric.


        :param name: The name of this Metric.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def current_value(self):
        """Gets the current_value of this Metric.  # noqa: E501


        :return: The current_value of this Metric.  # noqa: E501
        :rtype: float
        """
        return self._current_value

    @current_value.setter
    def current_value(self, current_value):
        """Sets the current_value of this Metric.


        :param current_value: The current_value of this Metric.  # noqa: E501
        :type: float
        """

        self._current_value = current_value

    @property
    def max_over_time(self):
        """Gets the max_over_time of this Metric.  # noqa: E501


        :return: The max_over_time of this Metric.  # noqa: E501
        :rtype: float
        """
        return self._max_over_time

    @max_over_time.setter
    def max_over_time(self, max_over_time):
        """Sets the max_over_time of this Metric.


        :param max_over_time: The max_over_time of this Metric.  # noqa: E501
        :type: float
        """

        self._max_over_time = max_over_time

    @property
    def min_over_time(self):
        """Gets the min_over_time of this Metric.  # noqa: E501


        :return: The min_over_time of this Metric.  # noqa: E501
        :rtype: float
        """
        return self._min_over_time

    @min_over_time.setter
    def min_over_time(self, min_over_time):
        """Sets the min_over_time of this Metric.


        :param min_over_time: The min_over_time of this Metric.  # noqa: E501
        :type: float
        """

        self._min_over_time = min_over_time

    @property
    def median_over_time(self):
        """Gets the median_over_time of this Metric.  # noqa: E501


        :return: The median_over_time of this Metric.  # noqa: E501
        :rtype: float
        """
        return self._median_over_time

    @median_over_time.setter
    def median_over_time(self, median_over_time):
        """Sets the median_over_time of this Metric.


        :param median_over_time: The median_over_time of this Metric.  # noqa: E501
        :type: float
        """

        self._median_over_time = median_over_time

    @property
    def rate(self):
        """Gets the rate of this Metric.  # noqa: E501


        :return: The rate of this Metric.  # noqa: E501
        :rtype: float
        """
        return self._rate

    @rate.setter
    def rate(self, rate):
        """Sets the rate of this Metric.


        :param rate: The rate of this Metric.  # noqa: E501
        :type: float
        """

        self._rate = rate

    @property
    def max_rate(self):
        """Gets the max_rate of this Metric.  # noqa: E501


        :return: The max_rate of this Metric.  # noqa: E501
        :rtype: float
        """
        return self._max_rate

    @max_rate.setter
    def max_rate(self, max_rate):
        """Sets the max_rate of this Metric.


        :param max_rate: The max_rate of this Metric.  # noqa: E501
        :type: float
        """

        self._max_rate = max_rate

    @property
    def min_rate(self):
        """Gets the min_rate of this Metric.  # noqa: E501


        :return: The min_rate of this Metric.  # noqa: E501
        :rtype: float
        """
        return self._min_rate

    @min_rate.setter
    def min_rate(self, min_rate):
        """Sets the min_rate of this Metric.


        :param min_rate: The min_rate of this Metric.  # noqa: E501
        :type: float
        """

        self._min_rate = min_rate

    @property
    def median_rate(self):
        """Gets the median_rate of this Metric.  # noqa: E501


        :return: The median_rate of this Metric.  # noqa: E501
        :rtype: float
        """
        return self._median_rate

    @median_rate.setter
    def median_rate(self, median_rate):
        """Sets the median_rate of this Metric.


        :param median_rate: The median_rate of this Metric.  # noqa: E501
        :type: float
        """

        self._median_rate = median_rate

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Metric):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Metric):
            return True

        return self.to_dict() != other.to_dict()



JOB_SUBMIT_EXAMPLE = """\
# Submit a job without a yaml file, this will inherit the environment from the workspace.
$ anyscale job submit --name my-job --wait -- python main.py
Output
(anyscale +1.0s) Submitting job with config JobConfig(name='my-job', image_uri=None, compute_config=None, env_vars=None, py_modules=None, cloud=None, project=None, ray_version=None, job_queue_config=None).
(anyscale +1.7s) Uploading local dir '.' to cloud storage.
(anyscale +2.6s) Including workspace-managed pip dependencies.
(anyscale +3.2s) Job 'my-job' submitted, ID: 'prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe'.
(anyscale +3.2s) View the job in the UI: https://console.anyscale.com/jobs/prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe
(anyscale +3.2s) Waiting for the job to run. Interrupting this command will not cancel the job.
(anyscale +3.5s) Waiting for job 'prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe' to reach target state SUCCEEDED, currently in state: STARTING
(anyscale +1m19.7s) Job 'prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe' transitioned from STARTING to SUCCEEDED
(anyscale +1m19.7s) Job 'prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe' reached target state, exiting

# Submit a job specified by a yaml file, this will run a job with the specified settings.
# See https://docs.anyscale.com/reference/job-api/#jobconfig for a list of all available fields.
$ cat job.yaml
name: my-job
entrypoint: python main.py
image_uri: anyscale/ray:2.44.1-slim-py312-cu128 # (Optional) Exclusive with `containerfile`.
# compute_config: # (Optional) An inline dictionary can also be provided.
#   head_node:
#     instance_type: m5.8xlarge
working_dir: . # (Optional) Use current working directory "." as the working_dir. Can be any local path or remote .zip file in cloud storage.
requirements: # (Optional) List of requirements files to install. Can also be a path to a requirements.txt.
  - emoji
max_retries: 0 # (Optional) Maximum number of times the job will be retried before being marked failed. Defaults to `1`.
$ anyscale job submit -f job.yaml
Output
(anyscale +1.0s) Submitting job with config JobConfig(name='my-job', image_uri='anyscale/ray:2.44.1-slim-py312-cu128', compute_config=None, env_vars=None, py_modules=None, py_executable=None, cloud=None, project=None, ray_version=None, job_queue_config=None).
(anyscale +4.2s) Uploading local dir '.' to cloud storage.
(anyscale +5.4s) Job 'my-job' submitted, ID: 'prodjob_ugk21r7pt6sbsxfuxn1an82x3e'.
(anyscale +5.4s) View the job in the UI: https://console.anyscale.com/jobs/prodjob_ugk21r7pt6sbsxfuxn1an82x3e
(anyscale +5.4s) Use `--wait` to wait for the job to run and stream logs.
"""

JOB_STATUS_EXAMPLE = """\
$ anyscale job status -n my-job
id: prodjob_6ntzknwk1i9b1uw1zk1gp9dbhe
name: my-job
state: STARTING
runs:
- name: raysubmit_ynxBVGT1SmzndiXL
  state: SUCCEEDED
"""

JOB_TERMINATE_EXAMPLE = """\
$ anyscale job terminate -n my-job
(anyscale +5.4s) Marked job 'my-job' for termination
(anyscale +5.4s) Query the status of the job with `anyscale job status --name my-job`.
"""

JOB_ARCHIVE_EXAMPLE = """\
$ anyscale job archive -n my-job
(anyscale +8.5s) Job prodjob_vzq2pvkzyz3c1jw55kl76h4dk1 is successfully archived.
"""

JOB_LOGS_EXAMPLE = """\
$ anyscale job logs -n my-job
2024-08-23 20:31:10,913 INFO job_manager.py:531 -- Runtime env is setting up.
hello world
"""

JOB_WAIT_EXAMPLE = """\
$ anyscale job wait -n my-job
(anyscale +5.7s) Waiting for job 'my-job' to reach target state SUCCEEDED, currently in state: STARTING
(anyscale +1m34.2s) Job 'my-job' transitioned from STARTING to SUCCEEDED
(anyscale +1m34.2s) Job 'my-job' reached target state, exiting
"""

JOB_LIST_EXAMPLE = """\
$ anyscale job list -n my-job
Output
View your Jobs in the UI at https://console.anyscale.com/jobs
JOBS:
NAME    ID                                    COST  PROJECT NAME    CLUSTER NAME                                    CURRENT STATE           CREATOR           ENTRYPOINT
my-job  prodjob_s9x4uzc5jnkt5z53g4tujb3y2e       0  default         cluster_for_prodjob_s9x4uzc5jnkt5z53g4tujb3y2e  SUCCESS                 doc@anyscale.com  python main.py
"""

JOB_TAGS_ADD_EXAMPLE = """\
$ anyscale job tags add --name my-job --tag owner=alice --tag env=staging
"""

JOB_TAGS_REMOVE_EXAMPLE = """\
$ anyscale job tags remove --name my-job --key owner --key env
"""

JOB_TAGS_LIST_EXAMPLE = """\
$ anyscale job tags list --name my-job
"""
JOB_QUEUE_LIST = """\
$ anyscale job-queue list
Output
JOB QUEUES:
             ID                 NAME              CLUSTER ID                      CREATOR ID             MAX CONCURRENCY    IDLE TIMEOUT SEC    CURRENT CLUSTER STATE
jq_h8fcze2qkr8wttuuvapi1hvyuc  queue_3  ses_cjr7uaf1yh2ue5uzvd11p24p4u  usr_we8x7d7u8hq8mj2488ed9x47n6          3                 5000               Terminated
jq_v5bx9z1sd4pbxasxhdms37j4gi  queue_2  ses_k86raeu6k1t6z1bvyejn3vblad  usr_we8x7d7u8hq8mj2488ed9x47n6         10                 5000               Terminated
jq_ni6hk66nt3194msr7hzzj9daun  queue_1  ses_uhb8a9gamtarz68kcurpjh86sa  usr_we8x7d7u8hq8mj2488ed9x47n6         10                 5000               Terminated
"""

JOB_QUEUE_STATUS = """\
$ anyscale job-queue status --id jq_h8fcze2qkr8wttuuvapi1hvyuc
Output
ID                            : jq_h8fcze2qkr8wttuuvapi1hvyuc
USER PROVIDED ID              : queue_3
NAME                          : queue_3
CURRENT JOB QUEUE STATE       : ACTIVE
EXECUTION MODE                : PRIORITY
MAX CONCURRENCY               : 3
IDLE TIMEOUT SEC              : 5000
CREATED AT                    : 2025-04-15 20:40:44
CREATOR ID                    : usr_we8x7d7u8hq8mj2488ed9x47n6
CREATOR EMAIL                 : test@anyscale.com
COMPUTE CONFIG ID             : cpt_8hzsv1t4jvb6kwjhfqbfjw5i6b
CURRENT CLUSTER STATE         : Terminated
CLUSTER ID                    : ses_cjr7uaf1yh2ue5uzvd11p24p4u
PROJECT ID                    : prj_7FWKGPGPaD3Q5mvk9zK2viBD
CLOUD ID                      : cld_kvedZWag2qA8i5BjxUevf5i7
TOTAL JOBS                    : 6
SUCCESSFUL JOBS               : 6
FAILED JOBS                   : 0
ACTIVE JOBS                   : 0
"""

JOB_QUEUE_UPDATE = """\
$ anyscale job-queue update --id jq_h8fcze2qkr8wttuuvapi1hvyuc --max-concurrency 5
Output
ID                            : jq_h8fcze2qkr8wttuuvapi1hvyuc
USER PROVIDED ID              : queue_3
NAME                          : queue_3
CURRENT JOB QUEUE STATE       : ACTIVE
EXECUTION MODE                : PRIORITY
MAX CONCURRENCY               : 5
IDLE TIMEOUT SEC              : 5000
CREATED AT                    : 2025-04-15 20:40:44
CREATOR ID                    : usr_we8x7d7u8hq8mj2488ed9x47n6
CREATOR EMAIL                 : test@anyscale.com
COMPUTE CONFIG ID             : cpt_8hzsv1t4jvb6kwjhfqbfjw5i6b
CURRENT CLUSTER STATE         : Terminated
CLUSTER ID                    : ses_cjr7uaf1yh2ue5uzvd11p24p4u
PROJECT ID                    : prj_7FWKGPGPaD3Q5mvk9zK2viBD
CLOUD ID                      : cld_kvedZWag2qA8i5BjxUevf5i7
TOTAL JOBS                    : 6
SUCCESSFUL JOBS               : 6
FAILED JOBS                   : 0
ACTIVE JOBS                   : 0
"""

JOB_QUEUE_TAGS_ADD_EXAMPLE = """\
$ anyscale job-queue tags add --name my-queue --tag team=data --tag priority=high
"""

JOB_QUEUE_TAGS_REMOVE_EXAMPLE = """\
$ anyscale job-queue tags remove --name my-queue --key team --key priority
"""

JOB_QUEUE_TAGS_LIST_EXAMPLE = """\
$ anyscale job-queue tags list --name my-queue --json
"""

SCHEDULE_APPLY_EXAMPLE = """\
$ anyscale schedule apply -n my-schedule -f my-schedule.yaml
(anyscale +0.5s) Applying schedule with config ScheduleConfig(job_config=JobConfig(name='my-schedule', image_uri=None, compute_config=None, env_vars=None, py_modules=None, cloud=None, project=None, ray_version=None, job_queue_config=None), cron_expression='0 0 * * * *', timezone='UTC').
(anyscale +2.3s) Uploading local dir '.' to cloud storage.
(anyscale +3.7s) Including workspace-managed pip dependencies.
(anyscale +4.9s) Schedule 'my-schedule' submitted, ID: 'cronjob_vrjrbwcnfjjid7fsld3sfkn8jz'.

$ cat my-schedule.yaml
timezone: local
cron_expression: 0 0 * * * *
job_config:
    name: my-job
    entrypoint: python main.py
    max_retries: 5
"""

SCHEDULE_LIST_EXAMPLE = """\
$ anyscale schedule list -n my-schedule
Output
+------------------------------------+-------------+---------------+-----------+-------------+------------------+------------+------------------+-----------------------+
| ID                                 | NAME        | DESCRIPTION   | PROJECT   | CRON        | NEXT TRIGGER     | TIMEZONE   | CREATOR          | LATEST EXECUTION ID   |
|------------------------------------+-------------+---------------+-----------+-------------+------------------+------------+------------------+-----------------------|
| cronjob_vrjrbwcnfjjid7fsld3sfkn8jz | my-schedule |               | default   | 0 0 * * * * | 2 hours from now | UTC        | doc@anyscale.com |                       |
+------------------------------------+-------------+---------------+-----------+-------------+------------------+------------+------------------+-----------------------+
"""

SCHEDULE_PAUSE_EXAMPLE = """\
$ anyscale schedule pause -n my-schedule
(anyscale +3.6s) Set schedule 'my-schedule' to state DISABLED
"""

SCHEDULE_RESUME_EXAMPLE = """\
$ anyscale schedule resume -n my-schedule
(anyscale +4.1s) Set schedule 'my-schedule' to state ENABLED
"""

SCHEDULE_STATUS_EXAMPLE = """\
$ anyscale schedule status -n my-schedule
id: cronjob_vrjrbwcnfjjid7fsld3sfkn8jz
name: my-schedule
state: ENABLED
"""

SCHEDULE_RUN_EXAMPLE = """\
$ anyscale schedule run -n my-schedule
(anyscale +2.5s) Triggered job for schedule 'my-schedule'.
"""

SCHEDULE_URL_EXAMPLE = """\
$ anyscale schedule url -n my-schedule
Output
(anyscale +2.3s) View your schedule at https://console.anyscale.com/scheduled-jobs/cronjob_7zj
"""

WORKSPACE_CREATE_EXAMPLE = """\
$ anyscale workspace_v2 create -f config.yml
(anyscale +2.7s) Workspace created successfully id: expwrk_jstjkv15a1vmle2j1t59s4bm35
(anyscale +3.9s) Applied dynamic requirements to workspace id: my-workspace
(anyscale +4.8s) Applied environment variables to workspace id: my-workspace

$ cat config.yml
name: my-workspace
idle_termination_minutes: 10
env_vars:
    HUMPDAY: WEDS
requirements: requirements.txt
"""

WORKSPACE_START_EXAMPLE = """\
$ anyscale workspace_v2 start --name my-workspace
(anyscale +5.8s) Starting workspace 'my-workspace'
"""

WORKSPACE_TERMINATE_EXAMPLE = """\
$ anyscale workspace_v2 terminate --name my-workspace
(anyscale +2.5s) Terminating workspace 'my-workspace'
"""

WORKSPACE_STATUS_EXAMPLE = """\
$ anyscale workspace_v2 status --name my-workspace
(anyscale +2.3s) STARTING
"""

WORKSPACE_WAIT_EXAMPLE = """\
$ anyscale workspace_v2 wait --name my-workspace --state RUNNING
(anyscale +2.5s) Waiting for workspace 'expwrk_jstjkv15a1vmle2j1t59s4bm35' to reach target state RUNNING, currently in state: RUNNING
(anyscale +2.8s) Workspace 'expwrk_jstjkv15a1vmle2j1t59s4bm35' reached target state, exiting
"""

WORKSPACE_SSH_EXAMPLE = """\
$ anyscale workspace_v2 ssh --name my-workspace
Authorized uses only. All activity may be monitored and reported.
Warning: Permanently added '[0.0.0.0]:5020' (ED25519) to the list of known hosts.
(base) ray@ip-10-0-24-60:~/default$
"""

WORKSPACE_RUN_COMMAND_EXAMPLE = """\
$ anyscale workspace_v2 run_command --name my-workspace "echo hello world"
Authorized uses only. All activity may be monitored and reported.
Warning: Permanently added '[0.0.0.0]:5020' (ED25519) to the list of known hosts.
hello world
"""

WORKSPACE_PULL_EXAMPLE = """\
$ anyscale workspace_v2 pull --name my-workspace --local-dir my-local
Warning: Permanently added '54.212.209.253' (ED25519) to the list of known hosts.
Authorized uses only. All activity may be monitored and reported.
Warning: Permanently added '[0.0.0.0]:5020' (ED25519) to the list of known hosts.
receiving incremental file list
created directory my-local
./
my-local/
my-local/main.py

sent 118 bytes  received 141 bytes  172.67 bytes/sec
total size is 0  speedup is 0.00
"""

WORKSPACE_PUSH_EXAMPLE = """\
$ anyscale workspace_v2 push --name my-workspace --local-dir my-local
Warning: Permanently added '52.10.22.124' (ED25519) to the list of known hosts.
Authorized uses only. All activity may be monitored and reported.
Warning: Permanently added '[0.0.0.0]:5020' (ED25519) to the list of known hosts.
sending incremental file list
my-local/
my-local/main.py

sent 188 bytes  received 39 bytes  151.33 bytes/sec
total size is 0  speedup is 0.00
"""

WORKSPACE_UPDATE_EXAMPLE = """\
$ anyscale workspace_v2 update expwrk_6l2ldwbu8299ympgltn725ciak --config-file config.yaml
(anyscale +2.3s) Workspace updated successfully id: expwrk_6l2ldwbu8299ympgltn725ciak
(anyscale +5.6s) Applied dynamic requirements to workspace id: expwrk_6l2ldwbu8299ympgltn725ciak
(anyscale +9.1s) Applied environment variables to workspace id: expwrk_6l2ldwbu8299ympgltn725ciak

$ cat config.yaml
name: my-workspace
idle_termination_minutes: 10
env_vars:
    HUMPDAY: WEDS
requirements: requirements.txt
"""

WORKSPACE_GET_EXAMPLE = """\
$ anyscale workspace_v2 get --name my-workspace
id: my-workspace-id
name: my-workspace
state: RUNNING
"""

WORKSPACE_LIST_EXAMPLE = """\
$ anyscale workspace_v2 list
Listing workspaces with:
• name              = <any>
• project           = <any>
• cloud             = <any>
• created_by_me     = False
• states            = <all>
• tags              = <none>
• include_archived  = False
• sort              = <none>
• mode              = interactive
• per-page limit    = 10
• max-items total   = all

View your Workspaces in the UI at https://console.anyscale.com/workspaces

NAME              STATE     PROJECT           CLOUD        CREATED BY              CREATED AT
my-workspace-1    RUNNING   proj_abc          cld_xyz      user@example.com        2024-01-15 10:30
my-workspace-2    STARTING  proj_abc          cld_xyz      user@example.com        2024-01-16 14:20
Fetched 2 workspaces.
"""

WORKSPACE_TAGS_ADD_EXAMPLE = """\
$ anyscale workspace_v2 tags add --name my-workspace --tag team=ml --tag env=prod
"""

WORKSPACE_TAGS_REMOVE_EXAMPLE = """\
$ anyscale workspace_v2 tags remove --name my-workspace --key team --key env
"""

WORKSPACE_TAGS_LIST_EXAMPLE = """\
$ anyscale workspace_v2 tags list --name my-workspace
Output
Tags:
KEY   VALUE
env   prod
team  ml
"""

MACHINE_POOL_CREATE_EXAMPLE = """\
$ anyscale machine-pool create --name can-testing
Machine pool can-testing has been created successfully (ID mp_8ogdz85mdwxb8a92yo44nn84ox).
"""

MACHINE_POOL_UPDATE_EXAMPLE = """\
$ anyscale machine-pool update --name can-testing --spec-file spec.yaml
Updated machine pool 'can-testing'.
"""

MACHINE_POOL_DESCRIBE_EXAMPLE = """\
$ anyscale machine-pool describe --name can-testing --mode machines
"""

MACHINE_POOL_DELETE_EXAMPLE = """\
$ anyscale machine-pool delete --name can-testing
Deleted machine pool 'can-testing'.
"""

MACHINE_POOL_LIST_EXAMPLE = """\
$ anyscale machine-pool list
MACHINE POOL       ID                             Clouds
can-testing        mp_8ogdz85mdwxb8a92yo44nn84ox
"""

MACHINE_POOL_ATTACH_EXAMPLE = """\
$ anyscale machine-pool attach --name can-testing --cloud my-cloud
Attached machine pool 'can-testing' to cloud 'my-cloud'.
"""

MACHINE_POOL_DETACH_EXAMPLE = """\
$ anyscale machine-pool detach --name can-testing --cloud my-cloud
Detached machine pool 'can-testing' from cloud 'my-cloud'.
"""

RESOURCE_QUOTAS_CREATE_EXAMPLE = """
$ anyscale resource-quota create -n my-resource-quota --cloud my-cloud --project my-project --user-email someone@myorg.com --num-instances 100 --num-cpus 1000 --num-gpus 50 --num-accelerators A10G 10 --num-accelerators A100-80G 0
(anyscale +2.5s) Name: my-resource-quota
Cloud name: my-cloud
Project name: my-project
User email: someone@myorg.com
Number of CPUs: 1000
Number of instances: 100
Number of GPUs: 50
Number of accelerators: {'A10G': 10, 'A100-80G': 0}
(anyscale +2.5s) Resource quota created successfully ID: rsq_abcdef
"""

RESOURCE_QUOTAS_LIST_EXAMPLE = """
$ anyscale resource-quota list --cloud my-cloud
Resource quotas:
ID       NAME               CLOUD ID    PROJECT ID  USER ID     IS ENABLED    CREATED AT    DELETED AT    QUOTA
rsq_123  resource-quota-1   cld_abcdef  prj_abcdef  usr_abcdef  True          09/11/2024                  {'num_accelerators': {'A100-80G': 0, 'A10G': 10},
                                                                                                           'num_cpus': 1000,
                                                                                                           'num_gpus': 50,
                                                                                                           'num_instances': 100}
rsq_456  resource-quota-2   cld_abcdef              usr_abcdef  True          09/10/2024                  {'num_accelerators': {}, 'num_cpus': None, 'num_gpus': None, 'num_instances': 2}
rsq_789  resource-quota-3   cld_abcdef                          False         09/05/2024                  {'num_accelerators': {'A10G': 1},
                                                                                                            'num_cpus': None,
                                                                                                            'num_gpus': None,
                                                                                                            'num_instances': None}
"""

RESOURCE_QUOTAS_ENABLE_EXAMPLE = """
$ anyscale resource-quota enable --id rsq_abcdef
(anyscale +1.2s) Enabled resource quota with ID rsq_abcdef successfully.
"""

RESOURCE_QUOTAS_DISABLE_EXAMPLE = """
$ anyscale resource-quota disable --id rsq_abcdef
(anyscale +1.4s) Disabled resource quota with ID rsq_abcdef successfully.
"""

RESOURCE_QUOTAS_DELETE_EXAMPLE = """
$ anyscale resource-quota delete --id rsq_abcdef
(anyscale +1.0s) Resource quota with ID rsq_abcdef deleted successfully.
"""

COMPUTE_CONFIG_CREATE_EXAMPLE = """
$ anyscale compute-config create -n my-compute-config -f compute_config.yaml
(anyscale +3.7s) Created compute config: 'my-compute-config:1'
(anyscale +3.7s) View the compute config in the UI: 'https://console.anyscale.com/v2/...'

$cat compute_config.yaml
head_node:
  instance_type: m5.8xlarge
worker_nodes:
- instance_type: m5.8xlarge
  min_nodes: 5
  max_nodes: 5
  market_type: ON_DEMAND # (Optional) Defaults to ON_DEMAND
- instance_type: g4dn.xlarge
  min_nodes: 1
  max_nodes: 10
  market_type: PREFER_SPOT # (Optional) Defaults to ON_DEMAND
min_resources: # (Optional) Defaults to no minimum.
  CPU: 1
  GPU: 1
max_resources: # (Optional) Defaults to no maximum.
  CPU: 6
  GPU: 1
"""

COMPUTE_CONFIG_GET_EXAMPLE = """
$ anyscale compute-config get -n my-compute-config
name: my-compute-config:1
id: cpt_buthu4glxj3azv4e287jad3ya3
config:
  cloud: aviary-prod-us-east-1
  head_node:
    instance_type: m5.8xlarge
    resources:
      CPU: 0
      GPU: 0
  worker_nodes:
  - instance_type: m5.8xlarge
    name: m5.8xlarge
    min_nodes: 5
    max_nodes: 5
    market_type: ON_DEMAND
  - instance_type: g4dn.xlarge
    name: g4dn.xlarge
    min_nodes: 1
    max_nodes: 10
    market_type: PREFER_SPOT
  min_resources:
    CPU: 1
    GPU: 1
  max_resources:
    CPU: 6
    GPU: 1
  enable_cross_zone_scaling: false
  flags: {}
"""

COMPUTE_CONFIG_ARCHIVE_EXAMPLE = """
$ anyscale compute-config archive -n my-compute-config
(anyscale +2.3s) Compute config is successfully archived.
"""

IMAGE_BUILD_EXAMPLE = """
$ anyscale image build -f my.Dockerfile -n my-image --ray-version 2.21.0
(anyscale +2.8s) Building image. View it in the UI: https://console.anyscale.com/v2/...
(anyscale +1m53.0s) Waiting for image build to complete. Elapsed time: 102 seconds.
(anyscale +1m53.0s) Image build succeeded.
Image built successfully with URI: anyscale/image/my-image:1

$ cat my.Dockerfile
FROM anyscale/ray:2.21.0-py39
RUN pip install --no-cache-dir pandas
"""

IMAGE_GET_EXAMPLE = """
$ anyscale image get -n my-image
uri: anyscale/image/my-image:1
status: SUCCEEDED
ray_version: 2.21.0
"""

IMAGE_LIST_EXAMPLE = """
$ anyscale image list --no-interactive --max-items 2
NAME         LATEST VERSION  LATEST URI                    CREATED BY           CREATED AT
my-image     3               anyscale/image/my-image:3     dhyey@anyscale.com   2024-12-01 18:42
workspace    1               anyscale/image/workspace:1    joshlee@anyscale.com 2024-11-09 09:15
"""

IMAGE_REGISTER_EXAMPLE = """
$ anyscale image register --image-uri docker.io/myrepo/image:v2 --name mycoolimage --ray-version 2.30.0
Image registered successfully with URI: anyscale/image/mycoolimage:1
"""

IMAGE_ARCHIVE_EXAMPLE = """
$ anyscale image archive --name my-old-image
(anyscale +0.5s) Successfully archived image.
Image 'my-old-image' archived successfully.
"""

AGGREGATED_INSTANCE_USAGE_DOWNLOAD_CSV_EXAMPLE = """\
$ anyscale aggregated-instance-usage download-csv --start-date 2024-10-01 --end-date 2024-10-03
(anyscale +0.5s) Downloading aggregated instance usage CSV...
Download complete! File saved to 'aggregated_instance_usage_2024-10-01_2024-10-03.zip'
0:00:01 100% ━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━ 79.4 kB / 79.4 kB Downloading aggregated instance usage CSV
(anyscale +2.2s) Downloaded CSV to aggregated_instance_usage_2024-10-01_2024-10-03.zip
"""

USER_BATCH_CREATE_EXAMPLE = """\
$ anyscale user batch-create --users-file users.yaml
(anyscale +0.5s) Creating users...
(anyscale +0.8s) 2 users created.

$ cat users_file.yaml
create_users:
  - name: name1
    email: test1@anyscale.com
    password: ''
    is_sso_user: false
    lastname: lastname1
    title: title1
  - name: name2
    email: test2@anyscale.com
    password: ''
    is_sso_user: false
    lastname: lastname2
    title: title2
"""

USER_LIST_EXAMPLE = """\
$ anyscale user list --page-size 5 --no-interactive
Output
USERS:
EMAIL                ID         NAME         ROLE         CREATED AT
owner@anyscale.com   usr_owner  Owner Name   owner        2024-01-01 12:00
service@anyscale.com           Service Bot  collaborator 2024-02-01 08:30
"""

USER_GET_EXAMPLE = """\
$ anyscale user get --email owner@anyscale.com --json
Output
{
  "name": "Owner Name",
  "email": "owner@anyscale.com",
  "created_at": "2024-01-01T12:00:00+00:00",
  "permission_level": "owner",
  "user_id": "usr_owner"
}
"""

ORGANIZATION_INVITATION_CREATE_EXAMPLE = """\
$ anyscale organization-invitation create --emails test1@anyscale.com,test2@anyscale.com
(anyscale +0.5s) Creating organization invitations...
(anyscale +1.7s) Organization invitations sent to: test1@anyscale.com, test2@anyscale.com
"""

ORGANIZATION_INVITATION_LIST_EXAMPLE = """\
$ anyscale organization-invitation list
ID             Email              Created At           Expires At
-------------  -----------------  -------------------  -------------------
orginv_abcedf  test@anyscale.com  11/25/2024 10:24 PM  12/02/2024 10:24 PM
"""

ORGANIZATION_INVITATION_DELETE_EXAMPLE = """\
$ anyscale organization-invitation delete --email test@anyscale.com
(anyscale +0.6s) Organization invitation for test@anyscale.com deleted.
"""


PROJECT_ADD_COLLABORATORS_EXAMPLE = """\
$ anyscale project add-collaborators --cloud cloud_name --project project_name --users-file collaborators.yaml
(anyscale +1.3s) Successfully added 3 collaborators to project project_name.
$ cat collaborators.yaml
collaborators:
  - email: "test1@anyscale.com"
    permission_level: "collaborator"
  - email: "test2@anyscale.com"
    permission_level: "readonly"
  - email: "test3@anyscale.com"
    permission_level: "owner"
"""


PROJECT_GET_EXAMPLE = """\
$ anyscale project get --id my-project-id
"""


PROJECT_LIST_EXAMPLE = """\
$ anyscale project list --include-defaults --non-interactive --max-items 2
"""


PROJECT_CREATE_EXAMPLE = """\
$ anyscale project create --name "my-project" --cloud "my-cloud-id"
"""


PROJECT_DELETE_EXAMPLE = """\
$ anyscale project delete --id project-id
"""


PROJECT_GET_DEFAULT_EXAMPLE = """\
$ anyscale project get-default --cloud my-cloud-id --json
"""


CLOUD_SETUP_K8S_AWS_EXAMPLE = """\
# Set up a Kubernetes cloud on AWS EKS
$ anyscale cloud setup --provider aws --region us-west-2 --name my-k8s-cloud \\
    --stack k8s --cluster-name my-eks-cluster --namespace anyscale-operator \\
    --debug --yes
Output
(anyscale +1.0s) Setting up Kubernetes cloud 'my-k8s-cloud' on AWS
(anyscale +1.5s) Required CLI tools are installed (kubectl, helm, aws)
(anyscale +2.3s) Using namespace: anyscale-operator
(anyscale +5.2s) Cluster discovered: arn:aws:eks:us-west-2:123456789012:cluster/my-eks-cluster
(anyscale +8.4s) Setting up AWS infrastructure...
(anyscale +1m12.3s) CloudFormation stack created
(anyscale +1m15.7s) Cloud registered with ID: cld_abc123
(anyscale +1m18.2s) Adding Anyscale Helm repository...
(anyscale +1m20.5s) Installing Anyscale operator...
(anyscale +1m25.8s) Generated Helm values file: anyscale-helm-values-aws-anyscale-operator-20241015_175602.yaml
(anyscale +2m45.3s) Helm installation completed successfully
(anyscale +2m45.4s) Kubernetes cloud 'my-k8s-cloud' setup completed successfully!
"""

CLOUD_SETUP_K8S_GCP_EXAMPLE = """\
# Set up a Kubernetes cloud on GCP GKE
$ anyscale cloud setup --provider gcp --region us-central1 --name my-gke-cloud \\
    --stack k8s --cluster-name my-gke-cluster --namespace anyscale-operator \\
    --project-id my-project-123 --yes
Output
(anyscale +1.0s) Setting up Kubernetes cloud 'my-gke-cloud' on GCP
(anyscale +1.5s) Required CLI tools are installed (kubectl, helm, gcloud, gsutil)
(anyscale +2.3s) Using namespace: anyscale-operator
(anyscale +5.8s) Cluster discovered: gke_my-project-123_us-central1_my-gke-cluster
(anyscale +9.2s) Setting up GCP infrastructure...
(anyscale +15.4s) Created GCS bucket: anyscale-k8s-my-gke-cloud-a1b2c3d4
(anyscale +18.7s) Created service account: anyscale-operator-e5f6g7h8@my-project-123.iam.gserviceaccount.com
(anyscale +25.3s) GCP resources created successfully
(anyscale +28.1s) Cloud registered with ID: cld_xyz789
(anyscale +30.5s) Installing Anyscale operator...
(anyscale +35.8s) Generated Helm values file: anyscale-helm-values-gcp-anyscale-operator-20241015_180215.yaml
(anyscale +1m55.2s) Helm installation completed successfully
(anyscale +1m55.3s) Kubernetes cloud 'my-gke-cloud' setup completed successfully!
"""

CLOUD_SETUP_K8S_CUSTOM_VALUES_EXAMPLE = """\
# Set up a Kubernetes cloud with a custom Helm values file path
$ anyscale cloud setup --provider aws --region us-west-2 --name my-k8s-cloud \\
    --stack k8s --cluster-name my-eks-cluster \\
    --values-file /path/to/custom-values.yaml --yes
Output
(anyscale +1.0s) Setting up Kubernetes cloud 'my-k8s-cloud' on AWS
...
(anyscale +1m25.8s) Generated Helm values file: /path/to/custom-values.yaml
...
"""

CLOUD_ADD_COLLABORATORS_EXAMPLE = """\
$ anyscale cloud add-collaborators --cloud cloud_name --users-file collaborators.yaml
(anyscale +1.3s) Successfully added 2 collaborators to cloud cloud_name.
$ cat collaborators.yaml
collaborators:
  - email: "test1@anyscale.com"
    permission_level: "collaborator"
  - email: "test2@anyscale.com"
    permission_level: "readonly"
"""


CLOUD_RESOURCE_CREATE_EXAMPLE = """\
$ anyscale cloud resource create --cloud my-cloud -f new-cloud-resource.yaml
Successfully created cloud resource my-new-resource in cloud my-cloud.

$ cat new-cloud-resource.yaml
name: my-new-resource
provider: AWS
compute_stack: VM
region: us-west-2
networking_mode: PUBLIC
object_storage:
  bucket_name: s3://my-bucket
file_storage:
  file_storage_id: fs-123
aws_config:
  vpc_id: vpc-123
  subnet_ids:
  - subnet-123
  security_group_ids:
  - sg-123
  anyscale_iam_role_id: arn:aws:iam::123456789012:role/anyscale-role-123
  cluster_iam_role_id: arn:aws:iam::123456789012:role/cluster-role-123
  memorydb_cluster_name: my-memorydb-cluster
"""

CLOUD_RESOURCE_DELETE_EXAMPLE = """\
$ anyscale cloud resource delete --cloud my-cloud --resource my-resource
Output
Please confirm that you would like to remove resource my-resource from cloud my-cloud. [y/N]: y
(anyscale +3.5s) Successfully removed resource my-resource from cloud my-cloud!
"""

CLOUD_GET_CLOUD_EXAMPLE = """\
$ anyscale cloud get --name my-cloud
name: my-cloud
id: cld_123
resources:
- cloud_resource_id: cldrsrc_123
  name: vm-aws-us-west-2
  provider: AWS
  compute_stack: VM
  region: us-west-2
  networking_mode: PUBLIC
  object_storage:
    bucket_name: s3://my-bucket
  file_storage:
    file_storage_id: fs-123
  aws_config:
    vpc_id: vpc-123
    subnet_ids:
    - subnet-123
    security_group_ids:
    - sg-123
    anyscale_iam_role_id: arn:aws:iam::123456789012:role/anyscale-role-123
    cluster_iam_role_id: arn:aws:iam::123456789012:role/cluster-role-123
    memorydb_cluster_name: my-memorydb-cluster
"""

CLOUD_STATUS_EXAMPLE = """\
$ anyscale cloud status --name my-cloud
name: my-cloud
id: cld_123
created_at: 2022-10-18 05:12:13.335803+00:00
is_default: true
resources:
- cloud_resource_id: cldrsrc_123
  name: vm-aws-us-west-2
  provider: AWS
  compute_stack: VM
  region: us-west-2
  networking_mode: PUBLIC
  operator_status: HEALTHY
  operator_status_details: All components running
  object_storage:
    bucket_name: s3://my-bucket
  aws_config:
    vpc_id: vpc-123
    subnet_ids:
    - subnet-123
"""

CLOUD_GET_DEFAULT_CLOUD_EXAMPLE = """\
$ anyscale cloud get-default
name: anyscale_v2_default_cloud
id: cld_abc
provider: AWS
region: us-west-2
created_at: 2022-10-18 05:12:13.335803+00:00
is_default: true
compute_stack: VM
"""

CLOUD_CONFIG_UPDATE_EXAMPLE = """\
$ anyscale cloud config update --cloud-id cloud_id --enable-log-ingestion --enable-system-cluster
--enable-log-ingestion is specified. [...] If you are sure you want to enable this feature, please type "consent": consent
Output
(anyscale +7.3s) Successfully updated log ingestion configuration for cloud, cloud_id to True
--enable-system-cluster is specified. [...] Are you sure you want to enable system cluster? [y/N]: y
Output
(anyscale +11.4s) Successfully enabled system cluster for cloud cloud_id

$ anyscale cloud config update --cloud-id cloud_id --spec-file iam.yaml
Output
(anyscale +2.1s) Successfully updated cloud configuration for cloud my-cloud (resource: cldrsrc_xyz123)

$ anyscale cloud config update --cloud-id cloud_id --resource shared-usw2 --spec-file iam.yaml
Output
(anyscale +2.1s) Successfully updated cloud configuration for cloud my-cloud (resource: cldrsrc_abc456)

$ anyscale cloud config update --cloud-id cloud_id --cloud-resource-id cldrsrc_xyz123 --spec-file iam.yaml
Output
(anyscale +2.1s) Successfully updated cloud configuration for cloud my-cloud (resource: cldrsrc_xyz123)
"""

CLOUD_TERMINATE_SYSTEM_CLUSTER_EXAMPLE = """\
$ anyscale cloud terminate-system-cluster --cloud-id cloud_id --wait
(anyscale +1.3s) Waiting for system cluster termination............
(anyscale +1m22.9s) System cluster for cloud 'cloud_id' is Terminated.
"""

SERVICE_ARCHIVE_EXAMPLE = """\
$ anyscale service archive --name my_service
"""

SERVICE_DELETE_EXAMPLE = """\
$ anyscale service delete --name my_service
"""

SERVICE_LIST_EXAMPLE = """\
$ anyscale service list --state running --sort -created_at
"""

SERVICE_TAGS_ADD_EXAMPLE = """\
$ anyscale service tags add --name my-service --tag team=platform --tag env=prod
"""

SERVICE_TAGS_REMOVE_EXAMPLE = """\
$ anyscale service tags remove --name my-service --key team --key env
"""

SERVICE_TAGS_LIST_EXAMPLE = """\
$ anyscale service tags list --name my-service --json
"""

# User Group Examples
USER_GROUP_LIST_EXAMPLE = """\
$ anyscale user-group list --max-items 50
ID            Name
------------  ----------------
ug_abc123     Engineering
ug_def456     Data Science
ug_ghi789     Platform Team
"""

USER_GROUP_GET_EXAMPLE = """\
$ anyscale user-group get --id ug_abc123
ID               ug_abc123
Name             Engineering
Organization ID  org_abc123
Created At       2025-01-15 10:30:00 UTC
Updated At       2025-01-15 10:30:00 UTC
"""

USER_GROUP_MEMBERSHIP_LIST_EXAMPLE = """\
$ anyscale user-group membership list
{
  "Engineering": [
    "alice@example.com",
    "charlie@example.com"
  ],
  "Data Science": [
    "bob@example.com"
  ]
}

# Save output to a file
$ anyscale user-group membership list --output memberships.json
"""

# Policy Examples
POLICY_SET_EXAMPLE = """\
# Set policy for a cloud
$ anyscale policy set --resource-type cloud --resource-id cld_abc123 -f policy.yaml
(anyscale +0.5s) Setting policy for cloud cld_abc123...
(anyscale +1.2s) Policy for cloud cld_abc123 has been updated.

# Set policy for your organization (--resource-id is not allowed)
$ anyscale policy set --resource-type organization -f org_policy.yaml
(anyscale +0.5s) Setting policy for organization your organization...
(anyscale +1.2s) Policy for organization your organization has been updated.

$ cat policy.yaml
bindings:
  - role_name: collaborator
    principals:
      - ug_abc123
  - role_name: readonly
    principals:
      - ug_def456
      - ug_ghi789
"""

POLICY_GET_EXAMPLE = """\
# Get policy for a cloud
$ anyscale policy get --resource-type cloud --resource-id cld_abc123
(anyscale +0.5s) Policy for cloud cld_abc123:
Role      Principal (User Group ID)  Process Status
--------  -------------------------  --------------
collaborator  ug_abc123              success
readonly  ug_def456                  success
readonly  ug_ghi789                  success

# Get policy for your organization (--resource-id is not allowed)
$ anyscale policy get --resource-type organization
(anyscale +0.5s) Policy for organization your organization:
Role      Principal (User Group ID)  Process Status
--------  -------------------------  --------------
owner     ug_admins                  success
collaborator  ug_developers          success
"""

POLICY_LIST_EXAMPLE = """\
$ anyscale policy list --resource-type cloud
(anyscale +0.6s) cloud: cld_abc123
Role      Principal (User Group ID)  Process Status
--------  -------------------------  --------------
collaborator  ug_abc123              success
readonly  ug_def456                  success

(anyscale +0.6s) cloud: cld_xyz789
Role      Principal (User Group ID)  Process Status
--------  -------------------------  --------------
collaborator  ug_ghi789              pending
"""

# SCIM Examples
SCIM_ENFORCE_GROUP_PERMISSIONS_EXAMPLE = """\
$ anyscale scim enforce-groups
╭─────────────────── ⚠️  Confirmation Required ───────────────────╮
│ WARNING: This is a destructive operation that cannot be undone. │
│                                                                  │
│ All role bindings on users will be removed.                      │
│ Role bindings on user groups and service accounts are unchanged. │
╰──────────────────────────────────────────────────────────────────╯
Do you want to proceed? [y/N]: y
(anyscale +1.5s) Starting SCIM permission migration...
(anyscale +2.0s) SCIM permission migration completed successfully.
"""

SCIM_LIST_USERS_PERMISSIONS_EXAMPLE = """\
# List all users' effective cloud & project permissions, plus org owners
$ anyscale scim list-users-permissions
(anyscale +0.6s) Listing users and their effective permissions...
{
  "organization_id": "org_p72",
  "org_owners": [
    {
      "user_email": "admin1@p72.com",
      "user_id": "usr_aaa"
    }
  ],
  "users": [
    {
      "clouds": [
        {
          "cloud_id": "cld_111",
          "cloud_name": "prod-cloud",
          "role": "collaborator",
          "projects": [
            {
              "project_id": "prj_111",
              "project_name": "prod-project",
              "role": "readonly"
            }
          ]
        }
      ],
      "user_email": "alice@p72.com",
      "user_id": "usr_abc"
    }
  ]
}
"""

USER_LIST_PERMISSIONS_EXAMPLE = """\
# List all users' effective cloud & project permissions
$ anyscale user list-permissions
(anyscale +0.6s) Listing users and their effective permissions...
{
  "organization_id": "org_p72",
  "org_owners": [
    {
      "user_email": "admin1@p72.com",
      "user_id": "usr_aaa"
    }
  ],
  "users": [
    {
      "clouds": [
        {
          "cloud_id": "cld_111",
          "cloud_name": "prod-cloud",
          "role": "collaborator",
          "projects": [
            {
              "project_id": "prj_111",
              "project_name": "prod-project",
              "role": "readonly"
            }
          ]
        }
      ],
      "user_email": "alice@p72.com",
      "user_id": "usr_abc"
    }
  ]
}

# List permissions for a specific user
$ anyscale user list-permissions --user-id usr_abc
(anyscale +0.4s) Listing users and their effective permissions...
{
  "organization_id": "org_p72",
  "org_owners": [
    {
      "user_email": "admin1@p72.com",
      "user_id": "usr_aaa"
    }
  ],
  "users": [
    {
      "clouds": [
        {
          "cloud_id": "cld_111",
          "cloud_name": "prod-cloud",
          "role": "collaborator"
        }
      ],
      "user_email": "alice@p72.com",
      "user_id": "usr_abc"
    }
  ]
}
"""



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class RedactionResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'organization_id': 'str',
        'started_at': 'datetime',
        'completed_at': 'datetime',
        'success': 'bool',
        'dry_run': 'bool',
        'total_tables_processed': 'int',
        'total_rows_examined': 'int',
        'total_rows_redacted': 'int',
        'table_results': 'list[TableRedactionResult]',
        'errors': 'list[str]'
    }

    attribute_map = {
        'organization_id': 'organization_id',
        'started_at': 'started_at',
        'completed_at': 'completed_at',
        'success': 'success',
        'dry_run': 'dry_run',
        'total_tables_processed': 'total_tables_processed',
        'total_rows_examined': 'total_rows_examined',
        'total_rows_redacted': 'total_rows_redacted',
        'table_results': 'table_results',
        'errors': 'errors'
    }

    def __init__(self, organization_id=None, started_at=None, completed_at=None, success=None, dry_run=None, total_tables_processed=None, total_rows_examined=None, total_rows_redacted=None, table_results=None, errors=None, local_vars_configuration=None):  # noqa: E501
        """RedactionResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._organization_id = None
        self._started_at = None
        self._completed_at = None
        self._success = None
        self._dry_run = None
        self._total_tables_processed = None
        self._total_rows_examined = None
        self._total_rows_redacted = None
        self._table_results = None
        self._errors = None
        self.discriminator = None

        self.organization_id = organization_id
        self.started_at = started_at
        self.completed_at = completed_at
        self.success = success
        self.dry_run = dry_run
        self.total_tables_processed = total_tables_processed
        self.total_rows_examined = total_rows_examined
        self.total_rows_redacted = total_rows_redacted
        self.table_results = table_results
        if errors is not None:
            self.errors = errors

    @property
    def organization_id(self):
        """Gets the organization_id of this RedactionResult.  # noqa: E501


        :return: The organization_id of this RedactionResult.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this RedactionResult.


        :param organization_id: The organization_id of this RedactionResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def started_at(self):
        """Gets the started_at of this RedactionResult.  # noqa: E501


        :return: The started_at of this RedactionResult.  # noqa: E501
        :rtype: datetime
        """
        return self._started_at

    @started_at.setter
    def started_at(self, started_at):
        """Sets the started_at of this RedactionResult.


        :param started_at: The started_at of this RedactionResult.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and started_at is None:  # noqa: E501
            raise ValueError("Invalid value for `started_at`, must not be `None`")  # noqa: E501

        self._started_at = started_at

    @property
    def completed_at(self):
        """Gets the completed_at of this RedactionResult.  # noqa: E501


        :return: The completed_at of this RedactionResult.  # noqa: E501
        :rtype: datetime
        """
        return self._completed_at

    @completed_at.setter
    def completed_at(self, completed_at):
        """Sets the completed_at of this RedactionResult.


        :param completed_at: The completed_at of this RedactionResult.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and completed_at is None:  # noqa: E501
            raise ValueError("Invalid value for `completed_at`, must not be `None`")  # noqa: E501

        self._completed_at = completed_at

    @property
    def success(self):
        """Gets the success of this RedactionResult.  # noqa: E501


        :return: The success of this RedactionResult.  # noqa: E501
        :rtype: bool
        """
        return self._success

    @success.setter
    def success(self, success):
        """Sets the success of this RedactionResult.


        :param success: The success of this RedactionResult.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and success is None:  # noqa: E501
            raise ValueError("Invalid value for `success`, must not be `None`")  # noqa: E501

        self._success = success

    @property
    def dry_run(self):
        """Gets the dry_run of this RedactionResult.  # noqa: E501


        :return: The dry_run of this RedactionResult.  # noqa: E501
        :rtype: bool
        """
        return self._dry_run

    @dry_run.setter
    def dry_run(self, dry_run):
        """Sets the dry_run of this RedactionResult.


        :param dry_run: The dry_run of this RedactionResult.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and dry_run is None:  # noqa: E501
            raise ValueError("Invalid value for `dry_run`, must not be `None`")  # noqa: E501

        self._dry_run = dry_run

    @property
    def total_tables_processed(self):
        """Gets the total_tables_processed of this RedactionResult.  # noqa: E501


        :return: The total_tables_processed of this RedactionResult.  # noqa: E501
        :rtype: int
        """
        return self._total_tables_processed

    @total_tables_processed.setter
    def total_tables_processed(self, total_tables_processed):
        """Sets the total_tables_processed of this RedactionResult.


        :param total_tables_processed: The total_tables_processed of this RedactionResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_tables_processed is None:  # noqa: E501
            raise ValueError("Invalid value for `total_tables_processed`, must not be `None`")  # noqa: E501

        self._total_tables_processed = total_tables_processed

    @property
    def total_rows_examined(self):
        """Gets the total_rows_examined of this RedactionResult.  # noqa: E501


        :return: The total_rows_examined of this RedactionResult.  # noqa: E501
        :rtype: int
        """
        return self._total_rows_examined

    @total_rows_examined.setter
    def total_rows_examined(self, total_rows_examined):
        """Sets the total_rows_examined of this RedactionResult.


        :param total_rows_examined: The total_rows_examined of this RedactionResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_rows_examined is None:  # noqa: E501
            raise ValueError("Invalid value for `total_rows_examined`, must not be `None`")  # noqa: E501

        self._total_rows_examined = total_rows_examined

    @property
    def total_rows_redacted(self):
        """Gets the total_rows_redacted of this RedactionResult.  # noqa: E501


        :return: The total_rows_redacted of this RedactionResult.  # noqa: E501
        :rtype: int
        """
        return self._total_rows_redacted

    @total_rows_redacted.setter
    def total_rows_redacted(self, total_rows_redacted):
        """Sets the total_rows_redacted of this RedactionResult.


        :param total_rows_redacted: The total_rows_redacted of this RedactionResult.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total_rows_redacted is None:  # noqa: E501
            raise ValueError("Invalid value for `total_rows_redacted`, must not be `None`")  # noqa: E501

        self._total_rows_redacted = total_rows_redacted

    @property
    def table_results(self):
        """Gets the table_results of this RedactionResult.  # noqa: E501


        :return: The table_results of this RedactionResult.  # noqa: E501
        :rtype: list[TableRedactionResult]
        """
        return self._table_results

    @table_results.setter
    def table_results(self, table_results):
        """Sets the table_results of this RedactionResult.


        :param table_results: The table_results of this RedactionResult.  # noqa: E501
        :type: list[TableRedactionResult]
        """
        if self.local_vars_configuration.client_side_validation and table_results is None:  # noqa: E501
            raise ValueError("Invalid value for `table_results`, must not be `None`")  # noqa: E501

        self._table_results = table_results

    @property
    def errors(self):
        """Gets the errors of this RedactionResult.  # noqa: E501


        :return: The errors of this RedactionResult.  # noqa: E501
        :rtype: list[str]
        """
        return self._errors

    @errors.setter
    def errors(self, errors):
        """Sets the errors of this RedactionResult.


        :param errors: The errors of this RedactionResult.  # noqa: E501
        :type: list[str]
        """

        self._errors = errors

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RedactionResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, RedactionResult):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional

from anyscale._private.sdk import sdk_command
from anyscale.compute_config._private.compute_config_sdk import PrivateComputeConfigSDK
from anyscale.compute_config.models import (
    ComputeConfigListResult,
    ComputeConfigType,
    ComputeConfigVersion,
)


_COMPUTE_CONFIG_SDK_SINGLETON_KEY = "compute_config_sdk"

_CREATE_EXAMPLE = """
import anyscale
from anyscale.compute_config.models import ComputeConfig, HeadNodeConfig, MarketType, WorkerNodeGroupConfig

single_deployment_compute_config = ComputeConfig(
    head_node=HeadNodeConfig(
        instance_type="m5.8xlarge",
    ),
    worker_nodes=[
        WorkerNodeGroupConfig(
            instance_type="m5.8xlarge",
            min_nodes=5,
            max_nodes=5,
        ),
        WorkerNodeGroupConfig(
            instance_type="m5.4xlarge",
            min_nodes=1,
            max_nodes=10,
            market_type=MarketType.SPOT,
        ),
    ],
)
full_name: str = anyscale.compute_config.create(single_deployment_compute_config, name="my-single-deployment-compute-config")

multi_deployment_compute_config = MultiResourceComputeConfig(
    configs=[
        ComputeConfig(
            cloud_resource="vm-aws-us-west-1",
            head_node=HeadNodeConfig(
                instance_type="m5.2xlarge",
            ),
            worker_nodes=[
                WorkerNodeGroupConfig(
                    instance_type="m5.4xlarge",
                    min_nodes=1,
                    max_nodes=10,
                ),
            ],
        ),
        ComputeConfig(
            cloud_resource="vm-aws-us-west-2",
            head_node=HeadNodeConfig(
                instance_type="m5.2xlarge",
            ),
            worker_nodes=[
                WorkerNodeGroupConfig(
                    instance_type="m5.4xlarge",
                    min_nodes=1,
                    max_nodes=10,
                ),
            ],
        )
    ]
)
full_name: str = anyscale.compute_config.create(multi_deployment_compute_config, name="my-multi-deployment-compute-config")
"""

_CREATE_ARG_DOCSTRINGS = {
    "config": "The config options defining the compute config.",
    "name": "The name of the compute config. This should *not* include a version tag. If a name is not provided, one will be automatically generated.",
}


@sdk_command(
    _COMPUTE_CONFIG_SDK_SINGLETON_KEY,
    PrivateComputeConfigSDK,
    doc_py_example=_CREATE_EXAMPLE,
    arg_docstrings=_CREATE_ARG_DOCSTRINGS,
)
def create(
    config: ComputeConfigType,
    *,
    name: Optional[str],
    _private_sdk: Optional[PrivateComputeConfigSDK] = None,
) -> str:
    """Create a new version of a compute config.

    Returns the full name of the registered compute config, including the version.
    """
    full_name, _ = _private_sdk.create_compute_config(config, name=name)  # type: ignore
    return full_name


_GET_EXAMPLE = """
import anyscale
from anyscale.compute_config.models import ComputeConfig

compute_config: ComputeConfig = anyscale.compute_config.get("my-compute-config")
"""
_GET_ARG_DOCSTRINGS = {
    "name": "The name of the compute config. This can inclue an optional version tag, i.e., 'name:version'. If no version tag is provided, the latest version will be returned.",
    "include_archived": "Whether to consider archived compute configs (defaults to False).",
    "cloud": "Cloud name to filter by when resolving the compute config by name. Useful when multiple compute configs with the same name exist across different clouds.",
}


@sdk_command(
    _COMPUTE_CONFIG_SDK_SINGLETON_KEY,
    PrivateComputeConfigSDK,
    doc_py_example=_GET_EXAMPLE,
    arg_docstrings=_GET_ARG_DOCSTRINGS,
)
def get(
    name: Optional[str],
    *,
    include_archived: bool = False,
    cloud: Optional[str] = None,
    _id: Optional[str] = None,
    _private_sdk: Optional[PrivateComputeConfigSDK] = None,
) -> ComputeConfigVersion:
    """Get the compute config with the specified name.

    The name can contain an optional version tag, i.e., 'name:version'.
    If no version is provided, the latest one will be returned.

    Args:
        name: Name of the compute config (can include version tag)
        include_archived: Whether to include archived configs
        cloud: Cloud name to filter by when resolving config name
        _id: Internal parameter for fetching by ID
        _private_sdk: Internal SDK instance
    """
    # NOTE(edoakes): I want to avoid exposing fetching by ID in the public API,
    # but it's needed for parity with the existing CLI. Therefore I am adding it
    # as a hidden private API that can be used like: (`name="", _id=id`).
    return _private_sdk.get_compute_config(  # type: ignore
        name=name or None, id=_id, cloud=cloud, include_archived=include_archived
    )


_ARCHIVE_EXAMPLE = """
import anyscale

anyscale.compute_config.archive(name="my-compute-config")
"""

_ARCHIVE_ARG_DOCSTRINGS = {"name": "Name of the compute config."}


@sdk_command(
    _COMPUTE_CONFIG_SDK_SINGLETON_KEY,
    PrivateComputeConfigSDK,
    doc_py_example=_ARCHIVE_EXAMPLE,
    arg_docstrings=_ARCHIVE_ARG_DOCSTRINGS,
)
def archive(
    name: Optional[str],
    *,
    _id: Optional[str] = None,
    _private_sdk: Optional[PrivateComputeConfigSDK] = None,
):
    """Archive a compute config and all of its versions.

    The name can contain an optional version, e.g., 'name:version'.
    If no version is provided, the latest one will be archived.

    Once a compute config is archived, its name will no longer be usable in the organization.
    """
    # NOTE(edoakes): I want to avoid exposing fetching by ID in the public API,
    # but it's needed for parity with the existing CLI. Therefore I am adding it
    # as a hidden private API that can be used like: (`name="", _id=id`).
    return _private_sdk.archive_compute_config(name=name or None, id=_id)  # type: ignore


_GET_DEFAULT_EXAMPLE = """
import anyscale
from anyscale.compute_config.models import ComputeConfigVersion

# Get the default compute config for the default cloud
default_config: ComputeConfigVersion = anyscale.compute_config.get_default()

# Get the default compute config for a specific cloud
default_config: ComputeConfigVersion = anyscale.compute_config.get_default(cloud="my-cloud")

# Get the default compute config for a specific cloud resource
default_config: ComputeConfigVersion = anyscale.compute_config.get_default(
    cloud="my-cloud",
    cloud_resource="my-cloud-resource"
)
"""

_GET_DEFAULT_ARG_DOCSTRINGS = {
    "cloud": "Name of the cloud. If not provided, uses the organization's default cloud.",
    "cloud_resource": "Name of the cloud resource. If not provided, uses the default cloud resource for the cloud.",
}


@sdk_command(
    _COMPUTE_CONFIG_SDK_SINGLETON_KEY,
    PrivateComputeConfigSDK,
    doc_py_example=_GET_DEFAULT_EXAMPLE,
    arg_docstrings=_GET_DEFAULT_ARG_DOCSTRINGS,
)
def get_default(
    *,
    cloud: Optional[str] = None,
    cloud_resource: Optional[str] = None,
    _private_sdk: Optional[PrivateComputeConfigSDK] = None,
) -> ComputeConfigVersion:
    """Get the default compute config for the specified cloud.

    Returns the default compute configuration that will be used when no compute config
    is explicitly specified for a workload.

    Args:
        cloud: Name of the cloud. If not provided, uses the organization's default cloud.
        cloud_resource: Name of the cloud resource. If not provided, uses the default
            cloud resource for the cloud.
        _private_sdk: Internal SDK instance.

    Returns:
        ComputeConfigVersion containing the default compute config.
    """
    return _private_sdk.get_default_compute_config(  # type: ignore
        cloud=cloud, cloud_resource=cloud_resource
    )


_LIST_EXAMPLE = """
import anyscale

# List all compute configs created by the current user
configs = anyscale.compute_config.list()

# List with filtering and sorting
configs = anyscale.compute_config.list(
    cloud_name="aws-prod",
    sort_by="created_at",
    sort_order="desc",
    max_items=10
)

# Access results and pagination token
for config in configs.results:
    print(f"{config.name} (version {config.version})")

# Fetch next page if available
if configs.next_token:
    next_page = anyscale.compute_config.list(
        cloud_name="aws-prod",
        sort_by="created_at",
        sort_order="desc",
        max_items=10,
        next_token=configs.next_token
    )
"""

_LIST_ARG_DOCSTRINGS = {
    "name": "Filter by compute config name.",
    "cloud_id": "Filter by cloud ID.",
    "cloud_name": "Filter by cloud name.",
    "sort_by": "Field to sort by. Options: 'name', 'created_at', 'last_modified_at'. Default: 'last_modified_at'",
    "sort_order": "Sort order. Options: 'asc' (ascending) or 'desc' (descending). Default: 'asc'",
    "max_items": "Maximum number of items to return per page. Default: 20",
    "next_token": "Pagination token for fetching the next page of results.",
    "include_shared": "Include compute configs shared with the user (not just those created by the user). Default: False",
}


@sdk_command(
    _COMPUTE_CONFIG_SDK_SINGLETON_KEY,
    PrivateComputeConfigSDK,
    doc_py_example=_LIST_EXAMPLE,
    arg_docstrings=_LIST_ARG_DOCSTRINGS,
)
def list(  # noqa: A001, PLR0913
    *,
    name: Optional[str] = None,
    cloud_id: Optional[str] = None,
    cloud_name: Optional[str] = None,
    sort_by: str = "last_modified_at",
    sort_order: str = "asc",
    max_items: int = 20,
    next_token: Optional[str] = None,
    include_shared: bool = False,
    _id: Optional[str] = None,
    _private_sdk: Optional[PrivateComputeConfigSDK] = None,
) -> ComputeConfigListResult:
    """List compute configurations with filtering, sorting, and pagination.

    Returns a ComputeConfigListResult object containing:
    - results: List of compute config objects
    - next_token: Token for fetching the next page (None if no more results)
    - count: Number of results in this page

    Args:
        name: Filter by compute config name
        cloud_id: Filter by cloud ID
        cloud_name: Filter by cloud name
        sort_by: Field to sort by ('name', 'created_at', 'last_modified_at')
        sort_order: Sort order ('asc' or 'desc')
        max_items: Maximum number of items to return per page
        next_token: Pagination token for next page
        include_shared: Include configs shared with the user
        _id: Internal parameter for fetching by ID
        _private_sdk: Internal SDK instance

    Returns:
        ComputeConfigListResult with 'results' (list), 'next_token' (str|None), and 'count' (int)

    Raises:
        ValueError: If both cloud_id and cloud_name are provided
    """
    if cloud_id and cloud_name:
        raise ValueError(
            "Only one of cloud_id or cloud_name can be provided, not both."
        )

    return _private_sdk.list_compute_configs(  # type: ignore
        name=name,
        id=_id,
        cloud_id=cloud_id,
        cloud_name=cloud_name,
        sort_by=sort_by,
        sort_order=sort_order,
        max_items=max_items,
        next_token=next_token,
        include_shared=include_shared,
    )
